# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
import logging




# ----- PANDAS ---------
# TODO : optimize implementation

FRAMES_HEADER=list()
for i in range(0,101):
    if i>=1 and i<10:
        FRAMES_HEADER.append ( "Frame00"+str(i))
    elif i>9 and i<100:
        FRAMES_HEADER.append ( "Frame0"+str(i))
    else:
        FRAMES_HEADER.append ( "Frame"+str(i))

def isColumnNameExist( dataframe, name):
    if name in dataframe.columns.values:
        logging.debug("column name :[%s] already in the dataFrame",name)

# ---- Spatio temporal parameters -----
def buid_df_descriptiveCycle1_1(analysis_outputStats):

    df_collection_L={"mean" : [], "std" : [], "median" : []}
    df_collection_R={"mean" : [], "std" : [], "median" : []}

    for key in analysis_outputStats.keys():

        label = key[0]
        context = key[1]

        if context == "Left":
            for typIt in ["mean","std","median"]:
                df_L = pd.DataFrame({ label : [ analysis_outputStats[label,context][str(typIt)] ],
                                         'EventContext': str(context),
                                         'Stats type' : str(typIt) })
                df_collection_L[typIt].append(df_L)


        if context == "Right":
            for typIt in ["mean","std","median"]:
                df_R = pd.DataFrame({ label : [ analysis_outputStats[label,context][str(typIt)] ],
                                   'EventContext': str(context),
                                   'Stats type': str(typIt)
                                   })
                df_collection_R[typIt].append(df_R)

    left_flag = len(df_collection_L["mean"])
    right_flag = len(df_collection_R["mean"])


    if left_flag:
        df_L_mean=df_collection_L["mean"][0]
        df_L_std=df_collection_L["std"][0]
        df_L_median=df_collection_L["median"][0]
        for i in range(1,len(df_collection_L["mean"])):
            df_L_mean=pd.merge(df_L_mean,df_collection_L["mean"][i],how='outer')
            df_L_std=pd.merge(df_L_std,df_collection_L["std"][i],how='outer')
            df_L_median=pd.merge(df_L_median,df_collection_L["median"][i],how='outer')

    if right_flag:
        df_R_mean=df_collection_R["mean"][0]
        df_R_std=df_collection_R["std"][0]
        df_R_median=df_collection_R["median"][0]
        for i in range(1,len(df_collection_R["mean"])):
            df_R_mean=pd.merge(df_R_mean,df_collection_R["mean"][i],how='outer')
            df_R_std=pd.merge(df_R_std,df_collection_R["std"][i],how='outer')
            df_R_median=pd.merge(df_R_median,df_collection_R["median"][i],how='outer')

    if left_flag and right_flag:
        DF = pd.concat([df_L_mean,df_L_std,df_L_median,
                        df_R_mean,df_R_std,df_R_median],ignore_index=True)
    elif left_flag and not right_flag:
        DF = pd.concat([df_L_mean,df_L_std,df_L_median],ignore_index=True)

    elif not left_flag and  right_flag:
        DF = pd.concat([df_R_mean,df_R_std,df_R_median],ignore_index=True)

    return DF


def buid_df_cycles1_1(analysis_outputStats):

    df_collection_L=[]
    df_collection_R=[]

    for key in analysis_outputStats.keys():
        label = key[0]
        context = key[1]
        n =len(analysis_outputStats[label,context]['values'])

        if context == "Left":
            df_L = pd.DataFrame({ label : analysis_outputStats[label,context]['values'],
                               "Cycle": range(0,n),
                               'EventContext': str(context)
                               })

            df_collection_L.append(df_L)


        if context == "Right":
            df_R = pd.DataFrame({ label : analysis_outputStats[label,context]['values'],
                               "Cycle": range(0,n),
                               'EventContext': str(context)
                               })

            df_collection_R.append(df_R)

    left_flag = len(df_collection_L)
    right_flag = len(df_collection_R)

    if left_flag:
        df_L=df_collection_L[0]
        for i in range(1,len(df_collection_L)):
            df_L=pd.merge(df_L,df_collection_L[i])

    if right_flag:
        df_R=df_collection_R[0]
        for i in range(1,len(df_collection_R)):
            df_R=pd.merge(df_R,df_collection_R[i])

    if left_flag and right_flag:
        DF = pd.concat([df_L,df_R],ignore_index=True)
    elif left_flag and not right_flag:
        DF = df_L
    elif not left_flag and  right_flag:
       DF = df_R

    return DF


# ---- for Scores -----
def buid_df_descriptiveCycle1_1_overall(analysis_outputStats,columnName):

    df_collection=[]

    valuesMean=analysis_outputStats["mean"]
    df=pd.DataFrame(valuesMean,  columns = [columnName])
    df['EventContext']= "overall"
    df['Stats type'] = "mean"
    df_collection.append(df)

    valuesStd=analysis_outputStats["std"]
    df=pd.DataFrame(valuesStd,  columns = [columnName])
    df['EventContext']= "overall"
    df['Stats type'] = "std"
    df_collection.append(df)

    valuesMedian=analysis_outputStats["median"]
    df=pd.DataFrame(valuesMedian,  columns = [columnName])
    df['EventContext']= "overall"
    df['Stats type'] = "median"
    df_collection.append(df)

    df=pd.concat(df_collection,ignore_index=True)

    return df



def buid_df_descriptiveCycle1_1_onlyContext(analysis_outputStats,columnName):


    # "data" section
    df_collection_L=[]
    df_collection_R=[]

    for context in analysis_outputStats.keys():

        if context == "Left":
            valuesMean=analysis_outputStats[context]["mean"]
            df=pd.DataFrame(valuesMean,  columns = [columnName])
            df['EventContext']= context
            df['Stats type'] = "mean"
            df_collection_L.append(df)

            valuesStd=analysis_outputStats[context]["std"]
            df=pd.DataFrame(valuesStd,  columns= [columnName])
            df['EventContext']= context
            df['Stats type'] = "std"
            df_collection_L.append(df)

            valuesMedian=analysis_outputStats[context]["median"]
            df=pd.DataFrame(valuesMedian,  columns= [columnName])
            df['EventContext']= context
            df['Stats type'] = "median"
            df_collection_L.append(df)


        if context == "Right":
            valuesMean=analysis_outputStats[context]["mean"]
            df=pd.DataFrame(valuesMean,  columns = [columnName])
            df['EventContext']= context
            df['Stats type'] = "mean"
            df_collection_R.append(df)

            valuesStd=analysis_outputStats[context]["std"]
            df=pd.DataFrame(valuesStd,  columns= [columnName])
            df['EventContext']= context
            df['Stats type'] = "std"
            df_collection_R.append(df)

            valuesMedian=analysis_outputStats[context]["median"]
            df=pd.DataFrame(valuesMedian,  columns= [columnName])
            df['EventContext']= context
            df['Stats type'] = "median"
            df_collection_R.append(df)

    left_flag = len(df_collection_L)
    right_flag = len(df_collection_R)

    if left_flag:  df_L=pd.concat(df_collection_L,ignore_index=True)
    if right_flag: df_R=pd.concat(df_collection_R,ignore_index=True)

    if left_flag and right_flag:
        DF = pd.concat([df_L,df_R],ignore_index=True)
    if left_flag and not right_flag:
        DF = df_L
    if not left_flag and right_flag:
        DF = df_R

    return DF


def buid_df_cycles1_1_onlyContext(analysis_outputStats, columnName):

    # "data" section
    df_collection_L=[]
    df_collection_R=[]

    for context in analysis_outputStats.keys():
        if context =="Left" : # FIXME TODO : if different form L and R
            values = analysis_outputStats[context]["values"]
            df=pd.DataFrame(values,  columns= [columnName])
            df['Cycle']= 0 if  len(values) == 1 else range(0,len(values))
            df['EventContext']= context
            df_collection_L.append(df)

        if context=="Right" :
            values = analysis_outputStats[context]["values"]
            df=pd.DataFrame(values,  columns= [columnName])
            df['Cycle']= 0 if  len(values) == 1 else range(0,len(values))
            df['EventContext']= context
            df_collection_R.append(df)

    left_flag = len(df_collection_L)
    right_flag = len(df_collection_R)

    if left_flag:
        df_L=pd.concat(df_collection_L,ignore_index=True)

    if right_flag:
        df_R=pd.concat(df_collection_R,ignore_index=True)

    if left_flag and right_flag:
        df_cycles = pd.concat([df_L,df_R],ignore_index=True)

    if left_flag and not right_flag:
        df_cycles = df_L

    if not left_flag and right_flag:
        df_cycles = df_R

    DF = df_cycles


    return DF




def buid_df_descriptiveCycle1_3(analysis_outputStats,columnName):


    # "data" section
    df_collection_L=[]
    df_collection_R=[]

    for key in analysis_outputStats.keys():
        label = key[0]
        context = key[1]
        if context == "Left":
            valuesMean=analysis_outputStats[label,context]["mean"]
            df=pd.DataFrame(valuesMean,  columns = [columnName])
            df['Axe']=['X','Y','Z']
            df['Label']=label
            df['EventContext']= context
            df['Stats type'] = "mean"
            df_collection_L.append(df)

            valuesStd=analysis_outputStats[label,context]["std"]
            df=pd.DataFrame(valuesStd,  columns= [columnName])
            df['Axe']=['X','Y','Z']
            df['Label']=label
            df['EventContext']= context
            df['Stats type'] = "std"
            df_collection_L.append(df)

            valuesMedian=analysis_outputStats[label,context]["median"]
            df=pd.DataFrame(valuesMedian,  columns= [columnName])
            df['Axe']=['X','Y','Z']
            df['Label']=label
            df['EventContext']= context
            df['Stats type'] = "median"
            df_collection_L.append(df)


        if context == "Right":
            valuesMean=analysis_outputStats[label,context]["mean"]
            df=pd.DataFrame(valuesMean,  columns= [columnName])
            df['Axe']=['X','Y','Z']
            df['Label']=label
            df['EventContext']= context
            df['Stats type'] = "mean"
            df_collection_R.append(df)

            valuesStd=analysis_outputStats[label,context]["std"]
            df=pd.DataFrame(valuesStd,  columns= [columnName])
            df['Axe']=['X','Y','Z']
            df['Label']=label
            df['EventContext']= context
            df['Stats type'] = "std"
            df_collection_R.append(df)

            valuesMedian=analysis_outputStats[label,context]["median"]
            df=pd.DataFrame(valuesMedian,  columns= [columnName])
            df['Axe']=['X','Y','Z']
            df['Label']=label
            df['EventContext']= context
            df['Stats type'] = "median"
            df_collection_R.append(df)

    left_flag = len(df_collection_L)
    right_flag = len(df_collection_R)

    if left_flag:  df_L=pd.concat(df_collection_L,ignore_index=True)
    if right_flag: df_R=pd.concat(df_collection_R,ignore_index=True)

    if left_flag and right_flag:
        DF = pd.concat([df_L,df_R],ignore_index=True)
    if left_flag and not right_flag:
        DF = df_L
    if not left_flag and right_flag:
        DF = df_R

    return DF


def buid_df_cycles1_3(analysis_outputStats, columnName):

    # "data" section
    df_collection_L=[]
    df_collection_R=[]

    for key in analysis_outputStats.keys():
        label = key[0]
        context = key[1]
        i_l=0
        i_r=0
        if context =="Left" : # FIXME TODO : if different form L and R
            for itValues in analysis_outputStats[label,context]["values"]:
                df=pd.DataFrame(itValues,  columns= [columnName])
                df['Axis']=['X','Y','Z']
                df['Label']=label
                df['Cycle']= i_l
                df['EventContext']= context
                df_collection_L.append(df)
                i_l+=1

        if context=="Right" :
            for itValues in analysis_outputStats[label,context]["values"]:
                df=pd.DataFrame(itValues,  columns= [columnName])
                df['Axis']=['X','Y','Z']
                df['Label']=label
                df['Cycle']= i_r
                df['EventContext']= context
                df_collection_R.append(df)
                i_r+=1

    left_flag = len(df_collection_L)
    right_flag = len(df_collection_R)

    if left_flag:
        df_L=pd.concat(df_collection_L,ignore_index=True)

    if right_flag:
        df_R=pd.concat(df_collection_R,ignore_index=True)

    if left_flag and right_flag:
        df_cycles = pd.concat([df_L,df_R],ignore_index=True)

    if left_flag and not right_flag:
        df_cycles = df_L

    if not left_flag and right_flag:
        df_cycles = df_R

    DF = df_cycles


    return DF


# ---- for 101-frames data -----
def buid_df_descriptiveCycle101_3(analysis_outputStats):


    # "data" section
    df_collection_L=[]
    df_collection_R=[]

    for key in analysis_outputStats.data.keys():
        label = key[0]
        context = key[1]
        if context == "Left":

            valuesMean=analysis_outputStats.data[label,context]["mean"]
            if not np.all(valuesMean==0):
                df=pd.DataFrame(valuesMean.T,  columns = FRAMES_HEADER)
                df['Axe']=['X','Y','Z']
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "mean"
                df_collection_L.append(df)

            valuesStd=analysis_outputStats.data[label,context]["std"]
            if not np.all(valuesStd==0):
                df=pd.DataFrame(valuesStd.T,  columns= FRAMES_HEADER)
                df['Axe']=['X','Y','Z']
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "std"
                df_collection_L.append(df)

            valuesMedian=analysis_outputStats.data[label,context]["median"]
            if not np.all(valuesMedian==0):
                df=pd.DataFrame(valuesMedian.T,  columns= FRAMES_HEADER)
                df['Axe']=['X','Y','Z']
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "median"
                df_collection_L.append(df)


        if context == "Right":

            valuesMean=analysis_outputStats.data[label,context]["mean"]
            if not np.all(valuesMean==0):
                df=pd.DataFrame(valuesMean.T,  columns= FRAMES_HEADER)
                df['Axe']=['X','Y','Z']
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "mean"
                df_collection_R.append(df)

            valuesStd=analysis_outputStats.data[label,context]["std"]
            if not np.all(valuesStd==0):
                df=pd.DataFrame(valuesStd.T,  columns= FRAMES_HEADER)
                df['Axe']=['X','Y','Z']
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "std"
                df_collection_R.append(df)

            valuesMedian=analysis_outputStats.data[label,context]["median"]
            if not np.all(valuesMedian==0):
                df=pd.DataFrame(valuesMedian.T,  columns= FRAMES_HEADER)
                df['Axe']=['X','Y','Z']
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "median"
                df_collection_R.append(df)

    left_flag = len(df_collection_L)
    right_flag = len(df_collection_R)

    if left_flag:  df_L=pd.concat(df_collection_L,ignore_index=True)
    if right_flag: df_R=pd.concat(df_collection_R,ignore_index=True)

    if left_flag and right_flag:
        DF = pd.concat([df_L,df_R],ignore_index=True)
    if left_flag and not right_flag:
        DF = df_L
    if not left_flag and right_flag:
        DF = df_R

    return DF




def buid_df_cycles101_3(analysis_outputStats):

    # "data" section
    df_collection_L=[]
    df_collection_R=[]

    for key in analysis_outputStats.data.keys():
        label = key[0]
        context = key[1]
        i_l=0
        i_r=0
        if context =="Left" :
            for itValues in analysis_outputStats.data[label,context]["values"]:
                if not np.all(itValues==0):
                    df=pd.DataFrame(itValues.T,  columns= FRAMES_HEADER)
                    df['Axis']=['X','Y','Z']
                    df['Label']=label
                    df['Cycle']= i_l
                    df['EventContext']= context
                    df_collection_L.append(df)
                i_l+=1 # will serve for merging with spt

        if context=="Right" :
            for itValues in analysis_outputStats.data[label,context]["values"]:
                if not np.all(itValues==0):
                    df=pd.DataFrame(itValues.T,  columns= FRAMES_HEADER)
                    df['Axis']=['X','Y','Z']
                    df['Label']=label
                    df['Cycle']= i_r
                    df['EventContext']= context
                    df_collection_R.append(df)
                i_r+=1 # will serve for merging with spt

    left_flag = len(df_collection_L)
    right_flag = len(df_collection_R)



    if left_flag:
        df_L=pd.concat(df_collection_L,ignore_index=True)

    if right_flag:
        df_R=pd.concat(df_collection_R,ignore_index=True)

    if left_flag and right_flag:
        df_cycles = pd.concat([df_L,df_R],ignore_index=True)

    if left_flag and not right_flag:
        df_cycles = df_L

    if not left_flag and right_flag:
        df_cycles = df_R


    # pst section
    if analysis_outputStats.pst !={}:
        df_collection_pst_L=[]
        df_collection_pst_R=[]

        for key in analysis_outputStats.pst.keys():
            label = key[0]
            context = key[1]
            n =len(analysis_outputStats.pst[label,context]['values'])

            if context == "Left":
                df_pst_L = pd.DataFrame({ label : analysis_outputStats.pst[label,context]['values'],
                                   "Cycle": range(0,n),
                                   'EventContext': str(context)
                                   })

                df_collection_pst_L.append(df_pst_L)

            if context == "Right":
                df_pst_R = pd.DataFrame({ label : analysis_outputStats.pst[label,context]['values'],
                                   "Cycle": range(0,n),
                                   'EventContext': str(context)
                                   })

                df_collection_pst_R.append(df_pst_R)

        if left_flag:
            df_pst_L=df_collection_pst_L[0]
            for i in range(1,len(df_collection_pst_L)):
                df_pst_L=pd.merge(df_pst_L,df_collection_pst_L[i])

        if right_flag:
            df_pst_R=df_collection_pst_R[0]
            for i in range(1,len(df_collection_pst_R)):
                df_pst_R=pd.merge(df_pst_R,df_collection_pst_R[i])

        if left_flag and right_flag:
            df_pst = pd.concat([df_pst_L,df_pst_R],ignore_index=True)
        if left_flag and not right_flag:
            df_pst = df_pst_L
        if not left_flag and right_flag:
            df_pst = df_pst_R


        # merging
        DF=pd.merge(df_cycles,df_pst, on=['Cycle','EventContext'],how='outer')

    else:
        DF = df_cycles


    return DF

def buid_df_descriptiveCycle101_1(analysis_outputStats):


    # "data" section
    df_collection_L=[]
    df_collection_R=[]

    for key in analysis_outputStats.data.keys():
        label = key[0]
        context = key[1]
        if context == "Left":

            valuesMean=analysis_outputStats.data[label,context]["mean"]
            if not np.all(valuesMean==0):
                df=pd.DataFrame(valuesMean.T,  columns = FRAMES_HEADER)
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "mean"
                df_collection_L.append(df)

            valuesStd=analysis_outputStats.data[label,context]["std"]
            if not np.all(valuesStd==0):
                df=pd.DataFrame(valuesStd.T,  columns= FRAMES_HEADER)
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "std"
                df_collection_L.append(df)

            valuesMedian=analysis_outputStats.data[label,context]["median"]
            if not np.all(valuesMedian==0):
                df=pd.DataFrame(valuesMedian.T,  columns= FRAMES_HEADER)
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "median"
                df_collection_L.append(df)


        if context == "Right":

            valuesMean=analysis_outputStats.data[label,context]["mean"]
            if not np.all(valuesMean==0):
                df=pd.DataFrame(valuesMean.T,  columns= FRAMES_HEADER)
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "mean"
                df_collection_R.append(df)

            valuesStd=analysis_outputStats.data[label,context]["std"]
            if not np.all(valuesStd==0):
                df=pd.DataFrame(valuesStd.T,  columns= FRAMES_HEADER)
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "std"
                df_collection_R.append(df)

            valuesMedian=analysis_outputStats.data[label,context]["median"]
            if not np.all(valuesMedian==0):
                df=pd.DataFrame(valuesMedian.T,  columns= FRAMES_HEADER)
                df['Label']=label
                df['EventContext']= context
                df['Stats type'] = "median"
                df_collection_R.append(df)

    left_flag = len(df_collection_L)
    right_flag = len(df_collection_R)

    if left_flag:  df_L=pd.concat(df_collection_L,ignore_index=True)
    if right_flag: df_R=pd.concat(df_collection_R,ignore_index=True)

    if left_flag and right_flag:
        DF = pd.concat([df_L,df_R],ignore_index=True)
    if left_flag and not right_flag:
        DF = df_L
    if not left_flag and right_flag:
        DF = df_R

    return DF


def buid_df_cycles101_1(analysis_outputStats):

    # "data" section
    df_collection_L=[]
    df_collection_R=[]

    for key in analysis_outputStats.data.keys():
        label = key[0]
        context = key[1]
        i_l=0
        i_r=0
        if context =="Left" :
            for itValues in analysis_outputStats.data[label,context]["values"]:
                if not np.all(itValues==0):
                    df=pd.DataFrame(itValues.T,  columns= FRAMES_HEADER)
                    df['Label']=label
                    df['Cycle']= i_l
                    df['EventContext']= context
                    df_collection_L.append(df)
                i_l+=1 # will serve for merging with spt

        if context=="Right" :
            for itValues in analysis_outputStats.data[label,context]["values"]:
                if not np.all(itValues==0):
                    df=pd.DataFrame(itValues.T,  columns= FRAMES_HEADER)
                    df['Label']=label
                    df['Cycle']= i_r
                    df['EventContext']= context
                    df_collection_R.append(df)
                i_r+=1 # will serve for merging with spt

    left_flag = len(df_collection_L)
    right_flag = len(df_collection_R)



    if left_flag:
        df_L=pd.concat(df_collection_L,ignore_index=True)

    if right_flag:
        df_R=pd.concat(df_collection_R,ignore_index=True)

    if left_flag and right_flag:
        df_cycles = pd.concat([df_L,df_R],ignore_index=True)

    if left_flag and not right_flag:
        df_cycles = df_L

    if not left_flag and right_flag:
        df_cycles = df_R


    # pst section
    if analysis_outputStats.pst !={}:
        df_collection_pst_L=[]
        df_collection_pst_R=[]

        for key in analysis_outputStats.pst.keys():
            label = key[0]
            context = key[1]
            n =len(analysis_outputStats.pst[label,context]['values'])

            if context == "Left":
                df_pst_L = pd.DataFrame({ label : analysis_outputStats.pst[label,context]['values'],
                                   "Cycle": range(0,n),
                                   'EventContext': str(context)
                                   })

                df_collection_pst_L.append(df_pst_L)

            if context == "Right":
                df_pst_R = pd.DataFrame({ label : analysis_outputStats.pst[label,context]['values'],
                                   "Cycle": range(0,n),
                                   'EventContext': str(context)
                                   })

                df_collection_pst_R.append(df_pst_R)

        if left_flag:
            df_pst_L=df_collection_pst_L[0]
            for i in range(1,len(df_collection_pst_L)):
                df_pst_L=pd.merge(df_pst_L,df_collection_pst_L[i])

        if right_flag:
            df_pst_R=df_collection_pst_R[0]
            for i in range(1,len(df_collection_pst_R)):
                df_pst_R=pd.merge(df_pst_R,df_collection_pst_R[i])

        if left_flag and right_flag:
            df_pst = pd.concat([df_pst_L,df_pst_R],ignore_index=True)
        if left_flag and not right_flag:
            df_pst = df_pst_L
        if not left_flag and right_flag:
            df_pst = df_pst_R


        # merging
        DF=pd.merge(df_cycles,df_pst, on=['Cycle','EventContext'],how='outer')

    else:
        DF = df_cycles


    return DF



# -*- coding: utf-8 -*-
import logging


def getPhases(dataStats):
    #phases
    phases = dict()

    # Left
    phases["Left","ST"] = [0,int(dataStats.pst["stancePhase","Left"]["mean"])]
    phases["Left","SW"] = [int(dataStats.pst["stancePhase","Left"]["mean"]),
                        100]

    phases["Left","DS1"] = [0,
                                    int(dataStats.pst["doubleStance1","Left"]["mean"])]

    phases["Left","SS"] = [int(dataStats.pst["doubleStance1","Left"]["mean"]),
                                    int(dataStats.pst["stancePhase","Left"]["mean"] - dataStats.pst["doubleStance2","Left"]["mean"]) ]
    phases["Left","eSS"] = [int(dataStats.pst["doubleStance1","Left"]["mean"]),
                                    int(dataStats.pst["doubleStance1","Left"]["mean"] + 1/3.0* dataStats.pst["simpleStance","Left"]["mean"])]
    phases["Left","mSS"] = [int(dataStats.pst["doubleStance1","Left"]["mean"] + 1/3.0* dataStats.pst["simpleStance","Left"]["mean"]),
                                    int(dataStats.pst["doubleStance1","Left"]["mean"] + 2/3.0* dataStats.pst["simpleStance","Left"]["mean"])]
    phases["Left","lSS"] = [int(dataStats.pst["doubleStance1","Left"]["mean"] + 2/3.0* dataStats.pst["simpleStance","Left"]["mean"]),
                                    int(dataStats.pst["stancePhase","Left"]["mean"] - dataStats.pst["doubleStance2","Left"]["mean"])]

    phases["Left","DS2"] = [int(dataStats.pst["stancePhase","Left"]["mean"] - dataStats.pst["doubleStance2","Left"]["mean"]),
                                    int(dataStats.pst["stancePhase","Left"]["mean"])]

    phases["Left","eSW"] = [int(dataStats.pst["stancePhase","Left"]["mean"]),
                                    int(dataStats.pst["stancePhase","Left"]["mean"] + 1/3.0* dataStats.pst["swingPhase","Left"]["mean"])]

    phases["Left","mSW"] = [int(dataStats.pst["stancePhase","Left"]["mean"] + 1/3.0* dataStats.pst["swingPhase","Left"]["mean"]),
                                    int(dataStats.pst["stancePhase","Left"]["mean"] + 2/3.0* dataStats.pst["swingPhase","Left"]["mean"])]

    phases["Left","lSW"] = [int(dataStats.pst["stancePhase","Left"]["mean"] + 2/3.0* dataStats.pst["swingPhase","Left"]["mean"]),
                                    100]

    # Right
    phases["Right","ST"] = [0,int(dataStats.pst["stancePhase","Right"]["mean"])]
    phases["Right","SW"] = [int(dataStats.pst["stancePhase","Right"]["mean"]),
                                    100]

    phases["Right","DS1"] = [0,
                                    int(dataStats.pst["doubleStance1","Right"]["mean"])]
    phases["Right","SS"] = [int(dataStats.pst["doubleStance1","Right"]["mean"]),
                                    int(dataStats.pst["stancePhase","Right"]["mean"] - dataStats.pst["doubleStance2","Right"]["mean"]) ]

    phases["Right","eSS"] = [int(dataStats.pst["doubleStance1","Right"]["mean"]),
                                    int(dataStats.pst["doubleStance1","Right"]["mean"] + 1/3.0* dataStats.pst["simpleStance","Right"]["mean"])]
    phases["Right","mSS"] = [int(dataStats.pst["doubleStance1","Right"]["mean"] + 1/3.0* dataStats.pst["simpleStance","Right"]["mean"]),
                                    int(dataStats.pst["doubleStance1","Right"]["mean"] + 2/3.0* dataStats.pst["simpleStance","Right"]["mean"])]
    phases["Right","lSS"] = [int(dataStats.pst["doubleStance1","Right"]["mean"] + 2/3.0* dataStats.pst["simpleStance","Right"]["mean"]),
                                    int(dataStats.pst["stancePhase","Right"]["mean"] - dataStats.pst["doubleStance2","Right"]["mean"])]

    phases["Right","DS2"] = [int(dataStats.pst["stancePhase","Right"]["mean"] - dataStats.pst["doubleStance2","Right"]["mean"]),
                                    int(dataStats.pst["stancePhase","Right"]["mean"])]

    phases["Right","eSW"] = [int(dataStats.pst["stancePhase","Right"]["mean"]),
                                    int(dataStats.pst["stancePhase","Right"]["mean"] + 1/3.0* dataStats.pst["swingPhase","Right"]["mean"])]

    phases["Right","mSW"] = [int(dataStats.pst["stancePhase","Right"]["mean"] + 1/3.0* dataStats.pst["swingPhase","Right"]["mean"]),
                                    int(dataStats.pst["stancePhase","Right"]["mean"] + 2/3.0* dataStats.pst["swingPhase","Right"]["mean"])]

    phases["Right","lSW"] = [int(dataStats.pst["stancePhase","Right"]["mean"] + 2/3.0* dataStats.pst["swingPhase","Right"]["mean"]),
                                    100]

    return phases


def getNumberOfCycle(analysisInstance,label,context):

    statSection = getAnalysisSection(analysisInstance,label,context)
    values = statSection.data[label,context]["values"]
    n_cycle = len(values)

    return n_cycle

def getValues(analysisInstance,label,context):

    statSection = getAnalysisSection(analysisInstance,label,context)
    values = statSection.data[label,context]["values"]

    return values



def isKeyExist(analysisInstance,label,context,exceptionMode = False):

    flag = False
    for item in  [analysisInstance.kinematicStats.data,analysisInstance.kineticStats.data,analysisInstance.emgStats.data]:
        if (label,context) in item.keys():
            flag = True
            break

    if not flag:
        if exceptionMode:
            raise Exception ("[pyCGM2] label [%s] - context [%s] doesn t find in the analysis instance"%(label,context))
        else:
            logging.error("[pyCGM2] label [%s] - context [%s] doesn t find in the analysis instance"%(label,context))

    return flag

def getAnalysisSection(analysisInstance,label,context):

    if (label,context) in analysisInstance.kinematicStats.data.keys():
        return analysisInstance.kinematicStats
    elif (label,context) in analysisInstance.kineticStats.data.keys():
        return analysisInstance.kineticStats
    elif (label,context) in analysisInstance.emgStats.data.keys():
        return analysisInstance.emgStats
    else:
        raise Exception ("[pyCGM2] label [%s] - context [%s] doesn t find in the analysis instance"%(label,context))



# -*- coding: utf-8 -*-
"""Nexus Operation : **2DofCalibration**

Calibration of the knee with the Calibration2Dof method (dynaKad like method).

The script considers all frames of the c3d and detects the side autmaticallt from ANK marker trajectories

:param -s, --side [string]: lower limb side ( choice: Left or Right )
:param -b, --beginFrame [int]:  manual selection of the first  frame
:param -e, --endFrame [int]:   manual selection of the last  frame


Examples:
    In the script argument box of a python nexus operation, you can edit:

    >>>  -side=Left -b=50 -e=100
    (Left knee calibration between frames 50 and 100)


"""
import os
import traceback
import logging
import argparse
import matplotlib.pyplot as plt
import numpy as np

# pyCGM2 settings
import pyCGM2
from pyCGM2 import log; log.setLoggingLevel(logging.INFO)

# vicon nexus
try:
    import ViconNexus
except:
    from viconnexusapi import ViconNexus

# pyCGM2 libraries
from pyCGM2 import enums
from pyCGM2.Utils import files
from pyCGM2.Nexus import nexusFilters, nexusUtils,nexusTools

from pyCGM2.Model import  modelFilters

from pyCGM2.Configurator import CgmArgsManager
from pyCGM2.Lib.CGM import  kneeCalibration

def main():

    parser = argparse.ArgumentParser(description='2Dof Knee Calibration')
    parser.add_argument('-s','--side', type=str, help="Side : Left or Right")
    parser.add_argument('-b','--beginFrame', type=int, help="begin frame")
    parser.add_argument('-e','--endFrame', type=int, help="end frame")

    args = parser.parse_args()
    NEXUS = ViconNexus.ViconNexus()
    NEXUS_PYTHON_CONNECTED = NEXUS.Client.IsConnected()


    if NEXUS_PYTHON_CONNECTED: # run Operation

        DATA_PATH, reconstructedFilenameLabelledNoExt = NEXUS.GetTrialName()

        reconstructFilenameLabelled = reconstructedFilenameLabelledNoExt+".c3d"

        logging.info( "data Path: "+ DATA_PATH )
        logging.info( "reconstructed file: "+ reconstructFilenameLabelled)

        # --------------------------SUBJECT -----------------------------------
        # Notice : Work with ONE subject by session
        subjects = NEXUS.GetSubjectNames()
        subject = nexusTools.getActiveSubject(NEXUS)
        logging.info(  "Subject name : " + subject  )

        # --------------------pyCGM2 MODEL - INIT ------------------------------
        model = files.loadModel(DATA_PATH,subject)
        logging.info("loaded model : %s" %(model.version ))


        if model.version == "CGM1.0":
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM1-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM1-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM1-pyCGM2.settings")

        elif model.version == "CGM1.1":
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM1_1-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM1_1-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM1_1-pyCGM2.settings")

        elif model.version == "CGM2.1":
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_1-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_1-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_1-pyCGM2.settings")

        elif model.version == "CGM2.2":
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_2-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_2-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_2-pyCGM2.settings")
        elif model.version == "CGM2.3":
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_3-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_3-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_3-pyCGM2.settings")
        elif model.version in  ["CGM2.4"]:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_4-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_4-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_4-pyCGM2.settings")

        elif model.version in  ["CGM2.5"]:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_5-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_5-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_5-pyCGM2.settings")

        else:
            raise Exception ("model version not found [contact admin]")

        # --------------------------SESSION INFOS ------------------------------------
        mpInfo,mpFilename = files.getMpFileContent(DATA_PATH,"mp.pyCGM2",subject)

        #  translators management
        if model.version in  ["CGM1.0"]:
            translators = files.getTranslators(DATA_PATH,"CGM1.translators")
        elif model.version in  ["CGM1.1"]:
            translators = files.getTranslators(DATA_PATH,"CGM1_1.translators")
        elif model.version in  ["CGM2.1"]:
            translators = files.getTranslators(DATA_PATH,"CGM2_1.translators")
        elif model.version in  ["CGM2.2"]:
            translators = files.getTranslators(DATA_PATH,"CGM2_2.translators")
        elif model.version in  ["CGM2.3"]:
            translators = files.getTranslators(DATA_PATH,"CGM2_3.translators")
        elif model.version in  ["CGM2.4"]:
            translators = files.getTranslators(DATA_PATH,"CGM2_4.translators")
        elif model.version in  ["CGM2.5"]:
            translators = files.getTranslators(DATA_PATH,"CGM2_5.translators")

        if not translators:
           translators = settings["Translators"]

        # btkAcq builder
        nacf = nexusFilters.NexusConstructAcquisitionFilter(DATA_PATH,reconstructedFilenameLabelledNoExt,subject)
        acq = nacf.build()

        # --------------------------MODEL PROCESSING----------------------------
        model,acqFunc,side = kneeCalibration.calibration2Dof(model,
            DATA_PATH,reconstructFilenameLabelled,translators,
            args.side,args.beginFrame,args.endFrame,None,forceBtkAcq=acq)

        # ----------------------SAVE-------------------------------------------
        files.saveModel(model,DATA_PATH,subject)
        logging.warning("model updated with a  %s knee calibrated with 2Dof method" %(side))

        # save mp
        files.saveMp(mpInfo,model,DATA_PATH,mpFilename)

        # ----------------------VICON INTERFACE-------------------------------------------
        #--- update mp
        nexusUtils.updateNexusSubjectMp(NEXUS,model,subject)

        if side == "Left":
            nexusTools.appendBones(NEXUS,subject,acqFunc,"LFE0", model.getSegment("Left Thigh"),OriginValues = acqFunc.GetPoint("LKJC").GetValues() )
        elif side == "Right":
            nexusTools.appendBones(NEXUS,subject,acqFunc,"RFE0", model.getSegment("Right Thigh"),OriginValues = acqFunc.GetPoint("RKJC").GetValues() )

        # --------------------------NEW MOTION FILTER - DISPLAY BONES---------
        scp=modelFilters.StaticCalibrationProcedure(model)
        if model.version in  ["CGM1.0","CGM1.1","CGM2.1","CGM2.2"]:
            modMotion=modelFilters.ModelMotionFilter(scp,acqFunc,model,enums.motionMethod.Determinist)
            modMotion.compute()

        elif model.version in  ["CGM2.3","CGM2.4"]:

            proximalSegmentLabel=str(side+" Thigh")
            distalSegmentLabel=str(side+" Shank")
            # Motion
            modMotion=modelFilters.ModelMotionFilter(scp,acqFunc,model,enums.motionMethod.Sodervisk)
            modMotion.segmentalCompute([proximalSegmentLabel,distalSegmentLabel])


        # -- add nexus Bones
        if side == "Left":
            nexusTools.appendBones(NEXUS,subject,acqFunc,"LFE1", model.getSegment("Left Thigh"),OriginValues = acqFunc.GetPoint("LKJC").GetValues() )
            logging.warning("offset %s" %(str(model.mp_computed["LeftKneeFuncCalibrationOffset"] )))
        elif side == "Right":
            nexusTools.appendBones(NEXUS,subject,acqFunc,"RFE1", model.getSegment("Right Thigh"),OriginValues = acqFunc.GetPoint("RKJC").GetValues() )
            logging.warning("offset %s" %(str(model.mp_computed["RightKneeFuncCalibrationOffset"] )))

    else:
        raise Exception("NO Nexus connection. Turn on Nexus")

if __name__ == "__main__":

    main()



# -*- coding: utf-8 -*-
import numpy as np

def measureNorm(acq,MarkerLabel1,MarkerLabel2,markerDiameter=0):
    mv1 = acq.GetPoint(MarkerLabel1).GetValues()
    mv2 = acq.GetPoint(MarkerLabel2).GetValues()

    norm = np.linalg.norm(mv1-mv2, axis=1)

    return norm.mean()-markerDiameter



# -*- coding: utf-8 -*-
"""Nexus Operation : **plotTemporalKinetics**

The script displays kinetics with time as x-axis

:param -ps, --pointSuffix [string]: suffix adds to the vicon nomenclature outputs

Examples:
    In the script argument box of a python nexus operation, you can edit:

    >>>  -ps=py
    (all points will be suffixed with py (LHipMoment_py))


"""
import logging
import argparse
import matplotlib.pyplot as plt
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)



# pyCGM2 settings
import pyCGM2
from pyCGM2 import log; log.setLoggingLevel(logging.INFO)

# vicon nexus
try:
    import ViconNexus
except:
    from viconnexusapi import ViconNexus

# pyCGM2 libraries
from pyCGM2 import enums
from pyCGM2.Lib import plot

from pyCGM2.Nexus import  nexusTools,nexusFilters
from pyCGM2.Utils import files

def main():

    plt.close("all")

    parser = argparse.ArgumentParser(description='CGM plot temporal Kinetics')
    parser.add_argument('-ps','--pointSuffix', type=str, help='suffix of model outputs')

    args = parser.parse_args()




    NEXUS = ViconNexus.ViconNexus()
    NEXUS_PYTHON_CONNECTED = NEXUS.Client.IsConnected()

    if NEXUS_PYTHON_CONNECTED:

        pointSuffix = args.pointSuffix
        # --------------------------INPUTS ------------------------------------
        DATA_PATH, modelledFilenameNoExt = NEXUS.GetTrialName()


        modelledFilename = modelledFilenameNoExt+".c3d"

        logging.info( "data Path: "+ DATA_PATH )
        logging.info( "file: "+ modelledFilename)


        # ----- Subject -----
        # need subject to find input files
        subjects = NEXUS.GetSubjectNames()
        subject = nexusTools.getActiveSubject(NEXUS)
        logging.info(  "Subject name : " + subject  )

        # --------------------pyCGM2 MODEL ------------------------------
        model = files.loadModel(DATA_PATH,subject)
        modelVersion = model.version

        # btkAcq builder
        nacf = nexusFilters.NexusConstructAcquisitionFilter(DATA_PATH,modelledFilenameNoExt,subject)
        acq = nacf.build()


        # --------------------pyCGM2 MODEL ------------------------------
        if model.m_bodypart in [enums.BodyPart.LowerLimb,enums.BodyPart.LowerLimbTrunk, enums.BodyPart.FullBody]:
            plot.plotTemporalKinetic(DATA_PATH, modelledFilename,"LowerLimb", pointLabelSuffix=pointSuffix,exportPdf=True,btkAcq=acq)
        # if model.m_bodypart in [enums.BodyPart.LowerLimbTrunk, enums.BodyPart.FullBody]:
        #     plot.plotTemporalKinetic(DATA_PATH, modelledFilename,"Trunk", pointLabelSuffix=pointSuffix,exportPdf=True)
        # if model.m_bodypart in [enums.BodyPart.UpperLimb, enums.BodyPart.FullBody]:
        #     pass # TODO plot upperlimb panel




    else:
        raise Exception("NO Nexus connection. Turn on Nexus")

if __name__ == "__main__":

    main()



# -*- coding: utf-8 -*-
"""Nexus Operation : **KalmanGapFilling**

Low dimensional Kalman smoother that fills gaps in motion capture marker trajectories

This repository is a  Python implementation of a gap filling algorithm
(http://dx.doi.org/10.1016/j.jbiomech.2016.04.016)
that smooths trajectories in low dimensional subspaces, together with a Python plugin for Vicon Nexus.
"""


import logging

import pyCGM2
# vicon nexus
try:
    import ViconNexus
except:
    from viconnexusapi import ViconNexus

from pyCGM2.Gap import gapFilling
from pyCGM2.Nexus import nexusTools,nexusFilters



def main():

    NEXUS = ViconNexus.ViconNexus()
    NEXUS_PYTHON_CONNECTED = NEXUS.Client.IsConnected()

    if NEXUS_PYTHON_CONNECTED: # run Operation

        DATA_PATH, filenameLabelledNoExt = NEXUS.GetTrialName()

        logging.info( "data Path: "+ DATA_PATH )
        logging.info( "file: "+ filenameLabelledNoExt)

        subject = nexusTools.getActiveSubject(NEXUS) #checkActivatedSubject(NEXUS,subjects)
        logging.info("Gap filling for subject %s"%(subject))

        # btkAcq builder
        nacf = nexusFilters.NexusConstructAcquisitionFilter(DATA_PATH,filenameLabelledNoExt,subject)
        acq = nacf.build()

        #acq = btkTools.smartReader(str(DATA_PATH+filenameLabelledNoExt+".c3d"))

        gfp =  gapFilling.LowDimensionalKalmanFilterProcedure()
        gff = gapFilling.GapFillingFilter(gfp,acq)
        gff.fill()

        filledAcq  = gff.getFilledAcq()
        filledMarkers  = gff.getFilledMarkers()

        for marker in filledMarkers:
            nexusTools.setTrajectoryFromAcq(NEXUS,subject,marker,filledAcq)

    else:
        raise Exception("NO Nexus connection. Turn on Nexus")

if __name__ == "__main__":


    main()



# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
import logging
import matplotlib.pyplot as plt
from collections import OrderedDict

from pyCGM2.Tools import exportTools
from pyCGM2.Signal.detect_peaks import detect_peaks
from pyCGM2.Math import derivation

from pyCGM2.Processing import analysisHandler
import re

class JointPatternFilter(object):
    """
    """

    def __init__(self, jointPatternProcedure, analysis):

        self.m_procedure = jointPatternProcedure
        self.m_analysis = analysis

    @classmethod
    def interpretCriteria(cls,criteria):
        primaries=list()
        secondaries=list()

        criterias = criteria.split("+")
        for it in criterias:
            if  "(" not in it:
                status = re.findall( "\[(.*?)\]" ,it )[0]
                ide = re.findall("\#(\d{0,100})",it)[0]
                primaries.append({'status': status, 'ide': int(ide)})
            else:
                bracket =  re.findall( "\((.*?)\)" ,it )[0]
                score = re.findall(",(\d{0,100})", bracket)[0]
                dict2 = {"score":int(score), "case" : list()}
                casesStr = re.findall("(.*?),",bracket)[0]
                cases = casesStr.split("|")
                for case in cases:
                    status = re.findall( "\[(.*?)\]" ,case )[0]
                    ide = re.findall("\#(\d{0,100})",case)[0]
                    dict2["case"].append( {'status': status, 'ide': int(ide)})
                secondaries.append(dict2)
        return primaries, secondaries


    def getValues(self):
        dataframeData = self.m_procedure.detectValue(self.m_analysis)
        return dataframeData

    def getPatterns(self,filter=False):
        data = self.m_procedure.detectValue(self.m_analysis)
        dataframePattern = self.m_procedure.detectPattern()

        if not filter:
            return dataframePattern
        else:

            filteredDataPattern = dataframePattern.loc[dataframePattern['Success'].isin(['TRUE','partial-primary','partial-secondary'])]

            return filteredDataPattern#[['Context','Plane' ,'Pattern']]



class XlsJointPatternProcedure(object):


    def __init__(self,xlsFiles,pointSuffix=None):

        self.pointSuffix = ("_"+pointSuffix)  if pointSuffix is not None else ""
        self.m_xls = pd.ExcelFile(xlsFiles)
        self.data = None
        self.patterns = None


    def _getFrameLimits(self,cyclePeriod,context,data):

        phases = analysisHandler.getPhases(data)

        if cyclePeriod == "CYCLE":
            frameLimits = [0, 100]
        elif cyclePeriod in ["ST","SW","DS1","SS","eSS","mSS","lSS","eSW","mSW","lSW"]:
            frameLimits = phases[context,cyclePeriod]
        elif ("->" in cyclePeriod):
            separator = cyclePeriod.find("->")
            before = cyclePeriod[:separator]
            after = cyclePeriod[separator+2:]
            frameLimits = [phases[context, before][0], phases[context, after][1]]
        return frameLimits

    def _applyStatus(self,value,limits,status):

        limits = limits.split(",")
        status = status.split(",")

        if value<float(limits[0]):
            return status[0]
        elif value>float(limits[0]) and value<float(limits[1]):
            return status[1]
        elif value>float(limits[1]):
            return status[2]


    def _applyMethod(self,values,method,iArgs,frame0):
        if method == "mean":
            val = np.mean(values)
        elif method == "range":
            val = np.abs( np.max(values) - np.min(values))
        elif method == "min":
            val = np.min(values)
        elif method == "max":
            val = np.max(values)
        elif method == "timing-find":
            arg = float(iArgs)
            f = values
            g = np.ones(len(values))*arg
            idx = np.argwhere(np.diff(np.sign(f - g)) != 0).reshape(-1) + 0

            # -- display curve
            # x = np.arange(0, len(values))
            # plt.plot( f, '-+')
            # plt.plot( g, '-')
            # plt.plot(x[idx], f[idx], 'ro')
            # plt.show()

            if idx.shape[0] == 0:
                val = "NA"
            else:
                val= frame0 + idx[0]


        elif method == "threshold":
            args = iArgs.split(",")
            n = len(values)
            count=0
            val = "False"
            for value in values:
                if args[1] == "greater":
                    if value > float(args[0]):
                        count+=1
                if args[1] == "lesser":
                    if value < float(args[0]):
                        count+=1

            percentage = (count/n)/100
            if percentage >= float(args[2]):
                val = "True"

        elif method == "getValue":
            index = int(iArgs)
            val = values[index]

        elif method == "timing-min":
            val = frame0+np.argmin(values)

        elif method == "timing-max":
            val = frame0+ np.argmax(values)

        elif method == "slope":
            val = values[-1] - values[0]


        elif method == "peak":
            args = iArgs.split(",")
            indexes = detect_peaks(values, mph=float(args[0]), mpd=float(args[1]),  show = False, valley=False)
            if indexes.shape[0]  > 2:
                val = "True"
            else:
                val = "False"
        else:
            val = "NA"
        return val




    def detectValue(self,analysis):

        xlsData = self.m_xls.parse("data")

        # init new collumns
        xlsData["Value"] = "NA"
        xlsData["ValueStatus"] = "NA"

        for index in xlsData.index:
            row = xlsData.iloc[index,:]

            context = row["Context"] # str
            variable = row["Variable"]+self.pointSuffix
            plan = int(row["Plan"])
            derivate = int(row["derivate"])
            cyclePeriod = row["CyclePeriod"]
            method = row["Method"]
            args = row["MethodArgument"]
            limits = row["Ranges"]
            status = row["status [lesser,within,greater]"]

            if row["Domain"] == "Kinematics":
                data = analysis.kinematicStats
            elif row["Domain"] == "Kinetics":
                data = analysis.kineticStats


            value = "NA"
            if data.data!={}:
                if (variable,context) in data.data:
                    dataArray = data.data[variable,context]["mean"]
                    frames = self._getFrameLimits(cyclePeriod,context,data)

                    if derivate == 0:
                        values  = dataArray[frames[0]:frames[1]+1,plan]
                    elif derivate == 1:
                        der = derivation.firstOrderFiniteDifference(dataArray,1)
                        values = der[frames[0]:frames[1]+1,plan]
                    elif derivate == 2:
                        der = derivation.secondOrderFiniteDifference(dataArray,1)
                        values = der[frames[0]:frames[1]+1,plan]

                    value = self._applyMethod(values,method,args,frames[0])
                else:
                    logging.warning("[pyGM2] :row (%i) -  key [%s,%s] not found in analysis data instance" %(index,variable,context) )


            if value is not "NA":
                if (value == "False" or value == "True"):
                    valueStatus = value
                else:
                    valueStatus = self._applyStatus(value,limits,status)
            else:
                valueStatus = "NA"

            xlsData.set_value(index,"Value", value)
            xlsData.set_value(index,"ValueStatus", valueStatus)

            self.data = xlsData

        return xlsData

    def detectPattern(self):

        xlsPatterns = self.m_xls.parse("patterns")
        xlsPatterns["Success"] = "NA"

        #index = 1
        for index in xlsPatterns.index:
            row = xlsPatterns.iloc[index,:]

            criteria = row["Criteria"]
            primaries,secondaries = JointPatternFilter.interpretCriteria(criteria)


            # criteria test
            flag1=True
            if primaries != []:
                count =0
                nb_primaries = len(primaries)
                for dict1It in primaries:
                    rowData = self.data[self.data["Id"] == dict1It["ide"]]
                    indexSelect = rowData.index[0]
                    if rowData["ValueStatus"][indexSelect] ==  dict1It["status"]:
                        count +=1
                if count == nb_primaries:
                    flag1 = True
                else:
                    flag1 = False


            if secondaries != []:
                flag2 = True
                i=0
                for it in secondaries:
                    score = it["score"]
                    count =0
                    for dict2It in it["case"]:
                        rowData = self.data[self.data["Id"] == dict2It["ide"]]
                        indexSelect = rowData.index[0]
                        if rowData["ValueStatus"][indexSelect] ==  dict2It["status"]:
                            count+=1
                    if count<score:
                        flag2 = False
                        break
                    i+=1
            # final
            if secondaries == []:
                if flag1:
                    xlsPatterns.set_value(index,"Success", "TRUE")
                else:
                    xlsPatterns.set_value(index,"Success", "FALSE")

            if primaries == []:
                if flag2:
                    xlsPatterns.set_value(index,"Success", "TRUE")
                else:
                    xlsPatterns.set_value(index,"Success", "FALSE")

            if primaries != [] and secondaries != []:
                if flag1 and flag2:
                    xlsPatterns.set_value(index,"Success", "TRUE")
                if flag1 and not flag2:
                    xlsPatterns.set_value(index,"Success", "partial-primary")
                if not flag1 and flag2:
                    xlsPatterns.set_value(index,"Success", "partial-secondary")
                if not flag1 and not flag2:
                    xlsPatterns.set_value(index,"Success", "FALSE")

        self.patterns = xlsPatterns

        return  self.patterns



# -*- coding: utf-8 -*-
import numpy as np

from pyCGM2.Math import numeric


class ScoreFilter(object):
    """
    """

    def __init__(self, scoreProcedure, analysis, normativeDataSet):

        self.m_score = scoreProcedure

        # construct normative data
        self.m_normativeData =  normativeDataSet.data

        self.m_analysis=analysis


    def compute(self):
        descriptiveGvsStats,descriptiveGpsStats_context,descriptiveGpsStats = self.m_score.compute(self.m_analysis,self.m_normativeData)
        self.m_analysis.setGps(descriptiveGpsStats,descriptiveGpsStats_context)
        self.m_analysis.setGvs(descriptiveGvsStats)


class CGM1_GPS(object):


    def __init__(self,pointSuffix=None):

        pointSuffix = ("_"+pointSuffix)  if (pointSuffix is not None) else ""

        matchingNormativeDataLabel = dict()

        matchingNormativeDataLabel["LPelvisAngles"+pointSuffix,"Left"] =  "PelvisAngles"
#        matchingNormativeDataLabel["RPelvisAngles"+pointSuffix,"Right"]=  "Pelvis.Angles"   # dont use. see richard`s articles
        matchingNormativeDataLabel["LHipAngles"+pointSuffix,"Left"]=  "HipAngles"
        matchingNormativeDataLabel["RHipAngles"+pointSuffix,"Right"]=  "HipAngles"
        matchingNormativeDataLabel["LKneeAngles"+pointSuffix,"Left"]=  "KneeAngles"
        matchingNormativeDataLabel["RKneeAngles"+pointSuffix,"Right"]=  "KneeAngles"
        matchingNormativeDataLabel["LAnkleAngles"+pointSuffix,"Left"]=  "AnkleAngles"
        matchingNormativeDataLabel["RAnkleAngles"+pointSuffix,"Right"]=  "AnkleAngles"
        matchingNormativeDataLabel["LFootProgressAngles"+pointSuffix,"Left"]=  "FootProgressAngles"
        matchingNormativeDataLabel["RFootProgressAngles"+pointSuffix,"Right"]=  "FootProgressAngles"

        axes={"PelvisAngles":[0,1,2],"HipAngles":[0,1,2],"KneeAngles":[0],"AnkleAngles":[0],"FootProgressAngles":[2]}   # tip is to use label from normative dataset


        self.matchingNormativeDataLabel = matchingNormativeDataLabel
        self.axes = axes

    def compute(self,analysis,normativeData):

        gvs = dict()

        nLeftCycles,nRightCycles = analysis.getKinematicCycleNumbers()

        # --- MAP ---
        # left cycles
        for label,context in self.matchingNormativeDataLabel.keys():
            matchingNormativeDataLabel = self.matchingNormativeDataLabel[label,context]
            if context == "Left":
                left_rms_local = np.zeros((nLeftCycles,3))
                for i in range(0,nLeftCycles):
                    values = analysis.kinematicStats.data[label, context]["values"][i]
                    valuesNorm = normativeData[matchingNormativeDataLabel]["mean"]

                    if valuesNorm.shape[0] == 51:
                        rms = numeric.rms(values[0:101:2]-valuesNorm,axis=0)
                    else:
                        rms = numeric.rms(values-valuesNorm,axis=0)

                    left_rms_local[i,:] = rms

                gvs[label,context]=left_rms_local


        # right cycles
        for label,context in self.matchingNormativeDataLabel.keys():
            matchingNormativeDataLabel = self.matchingNormativeDataLabel[label,context]
            if context == "Right":
                right_rms_local = np.zeros((nRightCycles,3))
                for i in range(0,nRightCycles):
                    values = analysis.kinematicStats.data[label, context]["values"][i]
                    valuesNorm = normativeData[matchingNormativeDataLabel]["mean"]

                    if valuesNorm.shape[0] == 51:
                        rms = numeric.rms(values[0:101:2]-valuesNorm,axis=0)
                    else:
                        rms = numeric.rms(values-valuesNorm,axis=0)

                    right_rms_local[i,:] = rms

                gvs[label,context]=right_rms_local



        # --- GPS ---

        # number of axis. 15 according articles ( left )
        n_axis =0
        for axis in self.axes:
            n_axis = n_axis + len(self.axes[axis])

        # computation of rms for left cycles
        left_rms_global = np.zeros((nLeftCycles,n_axis))
        for i in range(0,nLeftCycles):
            cumAxis =0
            for label,context in  gvs.keys():
                if context == "Left":
                    axisIndex = self.axes[self.matchingNormativeDataLabel[label,context]] # use of the tip here
                    for axis in axisIndex:
                        left_rms_global[i,cumAxis] = gvs[label,context][i,axis]
                        cumAxis+=1

        # computation of rms for right cycles
        right_rms_global = np.zeros((nRightCycles,n_axis))
        for i in range(0,nRightCycles):
            cumAxis =0
            for label,context in  gvs.keys():
                if context == "Right":
                    axisIndex = self.axes[self.matchingNormativeDataLabel[label,context]]
                    for axis in axisIndex:
                        right_rms_global[i,cumAxis] = gvs[label,context][i,axis]
                        cumAxis+=1


        # output dictionnary
        outDict_gvs = dict()
        outDict_gps_context = dict()
        outDict_gps = dict()

        for label,context in self.matchingNormativeDataLabel.keys():
            outDict_gvs[label,context]={'mean':np.mean(gvs[label, context],axis=0),
                                          'std':np.std(gvs[label, context],axis=0),
                                          'median': np.median(gvs[label, context],axis=0),
                                          'values':gvs[label,context]}
        for context in ["Left","Right"]:
            gpsValues = left_rms_global if context == "Left" else right_rms_global
            # construction of the global dictionnary outputs
            outDict_gps_context[context]={'mean':np.array([np.mean(gpsValues.mean(axis=1))]),
                                      'std':np.array([np.std(gpsValues.mean(axis=1))]),
                                       'median': np.array([np.median(gpsValues.mean(axis=1))]),
                                       'values': gpsValues.mean(axis=1)}


        overall_gps_values = np.concatenate((outDict_gps_context["Right"]["values"],outDict_gps_context["Left"]["values"]))


        outDict_gps={'mean':np.array([np.mean(overall_gps_values)]),
                          'std':np.array([np.std(overall_gps_values)]),
                           'median': np.array([np.median(overall_gps_values)]),
                           'values': overall_gps_values}

        return outDict_gvs, outDict_gps_context,outDict_gps


class GDI(object):
    def __init__(self):
        self.name = "Gait Deviation index"

    def compute(self):
        pass



# -*- coding: utf-8 -*-
from __future__ import print_function
import logging
import os
import ConfigParser

import pyCGM2
from pyCGM2 import enums

from pyCGM2.ForcePlates import forceplates
from pyCGM2.Tools import btkTools
from pyCGM2.Utils import files,utils
from bs4 import BeautifulSoup



DEFAULT_SUBSESSION = {"Task":"","Shoes":"","ProthesisOrthosis":"","ExternalAid":"","PersonalAid":""}


def generateEmptyENF(path):
    c3ds = files.getFiles(path,"c3d")
    for c3d in c3ds:
        enfName = c3d[:-4]+".Trial.enf"
        if enfName not in os.listdir(path):
            open((path+enfName), 'a').close()


def getCurrentMarkedEnfs():
    currentMarkedNodesFile = os.getenv("PUBLIC")+"\\Documents\\Vicon\\Eclipse\\CurrentlyMarkedNodes.xml"

    infile = open(currentMarkedNodesFile,"r")
    soup = BeautifulSoup(infile.read(),'xml')

    out=list()
    nodes = soup.find_all("MarkedNode")
    for node in nodes:
        fullFilename = node.get("MarkedNodePath")
        out.append(fullFilename.split("\\")[-1])

    return out if out!=[] else None


def getEnfFiles(path, type):
    path = path[:-1] if path[-1:]=="\\" else path

    enfFiles = files.getFiles(path+"\\",type.value)

    if type == enums.EclipseType.Session:
        if len(enfFiles)>1:
            raise Exception ("Vicon Eclipse badly configured. Two session enf found")
        else:
            return  enfFiles[0]

    elif type == enums.EclipseType.Patient:
        if len(enfFiles)>1:
            raise Exception ("Vicon Eclipse badly configured. Two Patient enf found")
        else:
            return  enfFiles[0]
    elif type == enums.EclipseType.Trial:
        return enfFiles
    else:
        raise Exception ("eclipse file type not recognize. Shoud be an item of enums.eClipseType")


def findCalibration(path, ignoreSelect=False):
    enfs = getEnfFiles(path,enums.EclipseType.Trial)


    detected = list()
    for enf in enfs:
        enfTrial = TrialEnfReader(path,enf)
        if not ignoreSelect:
            if enfTrial.isCalibrationTrial() and enfTrial.isSelected() :
                detected.append(enf)
        else:
            if enfTrial.isCalibrationTrial() and enfTrial.get("Processing")=="Ready":
                detected.append(enf)

    if len(detected)>1:
        raise Exception("You should have only one activated calibration c3d")
    else:
        if detected ==[] : raise Exception("No static file detected")
        return detected[0]


def findMotions(path,ignoreSelect=False):
    enfs = getEnfFiles(path,enums.EclipseType.Trial)

    detected = list()
    for enf in enfs:
        enfTrial = TrialEnfReader(path,enf)
        if not ignoreSelect:
            if enfTrial.isMotionTrial() and enfTrial.isSelected():
                detected.append(enf)
        else:
            if enfTrial.isMotionTrial() and enfTrial.get("Processing")=="Ready":
                detected.append(enf)

    if detected ==[]:
        raise Exception("No motion files detected")
    else:
        return detected

def findKneeMotions(path,ignoreSelect=False):
    enfs = getEnfFiles(path,enums.EclipseType.Trial)

    detected = list()
    for enf in enfs:
        enfTrial = TrialEnfReader(path,enf)
        if not ignoreSelect:
            if enfTrial.isKneeCalibrationTrial() and enfTrial.isSelected():
                detected.append(enf)
        else:
            if enfTrial.isKneeCalibrationTrial() and enfTrial.get("Processing")=="Ready":
                detected.append(enf)

    if detected ==[]:
        return None
    else:
        return detected


def cleanEnf(path,enf):
    src = open((path+enf),"r")
    filteredContent = ""
    for line in src:
        if line[0] != "=":
           filteredContent += line
    src.close()

    dst = open((path+enf), 'w')
    dst.write(filteredContent)
    dst.close()
    return filteredContent


def classifyEnfMotions(path,ignoreSelect=False,
    criteria=["Task","Shoes","ProthesisOrthosis","ExternalAid","PersonalAid"]):
    """


    """
    enfs = getEnfFiles(path,enums.EclipseType.Trial)
    subSessions = list()

    # 1 : detect task

    for enf in enfs:
        subSession =dict(DEFAULT_SUBSESSION)
        enfTrial = TrialEnfReader(path,enf)
        if not ignoreSelect:
            if enfTrial.isMotionTrial() and enfTrial.isSelected():
                for it in criteria:
                    subSession[it] = enfTrial.get(it)
                if subSession not in subSessions:
                    subSessions.append(subSession)
        else:
            if enfTrial.isMotionTrial() and enfTrial.get("Processing")=="Ready":
                for it in criteria:
                    subSession[it] = enfTrial.get(it)
                if subSession not in subSessions:   subSessions.append(subSession)


    #2 : match c3d
    c3ds = list()
    for subSessionIt in subSessions:
        c3d_bySubSession = list()
        for enf in enfs:
            subSession =dict(DEFAULT_SUBSESSION)
            enfTrial = TrialEnfReader(path,enf)
            if not ignoreSelect:
                if enfTrial.isMotionTrial() and enfTrial.isSelected():
                    for it in criteria:
                        subSession[it] = enfTrial.get(it)
                    if subSession == subSessionIt:
                        c3d_bySubSession.append(enfTrial.getFile())
            else:
                if enfTrial.isMotionTrial() and enfTrial.get("Processing")=="Ready":
                    for it in criteria:
                        subSession[it] = enfTrial.get(it)
                    if subSession == subSessionIt:
                        c3d_bySubSession.append(enfTrial.getFile())
        c3ds.append(c3d_bySubSession)

    shortNames = list()
    for subSessionIt in subSessions:
        shortNames.append(subSessionIt["Task"][0] +"_"+ subSessionIt["Shoes"][0]+ subSessionIt["ProthesisOrthosis"][0]+ subSessionIt["ExternalAid"][0]+ subSessionIt["PersonalAid"][0])


    # zipped and retrun list of ClassifiedEnf object
    zipped = zip(subSessions,c3ds,shortNames )
    out = list()
    for it in zipped:
        out.append(ClassifiedEnf(it[0],it[1],it[2]))

    return out







class EnfReader(object):

    def __init__(self, path,enfFile):

        config = ConfigParser.ConfigParser()
        config.optionxform=str # keep letter case

        if not os.path.isfile((path+enfFile)):
            raise Exception ("[pyCGM2] : enf file (%s) not find"%(utils.str(path+enfFile)))

        try:
            config.read((path+enfFile))
        except ConfigParser.ParsingError:
            cleanEnf(path,enfFile)
            config.read((path+enfFile))


        self.m_path =  path
        self.m_file =  enfFile
        self.m_config = config

    def getSection(self,section):
        dict1 = {}
        options = self.m_config.options(section)
        for option in options:
            try:
                dict1[option] = self.m_config.get(section, option)
                if dict1[option] == -1:
                    print("skip: %s" % option)
            except:
                print("exception on %s!" % option)
                dict1[option] = None
        return dict1

    def getFile(self):
        return self.m_file

    def getPath(self):
        return self.m_path



class PatientEnfReader(EnfReader):

    def __init__(self, path,enfFile):
        super(PatientEnfReader, self).__init__(path,enfFile)
        self.m_patientInfos = super(PatientEnfReader, self).getSection("SUBJECT_INFO")

        for key in self.m_patientInfos:
            if self.m_patientInfos[key] == "":
                self.m_patientInfos[key] = None
            elif self.m_patientInfos[key].lower() == "true":
                self.m_patientInfos[key]=True
            elif self.m_patientInfos[key].lower() == "false":
                self.m_patientInfos[key]=False
            else:
                pass

    def get(self,label):
        if label in self.m_patientInfos.keys():
            return self.m_patientInfos[label]

    def getPatientInfos(self):
        return self.m_patientInfos



class SessionEnfReader(EnfReader):

    def __init__(self, path,enfFile):
        super(SessionEnfReader, self).__init__(path,enfFile)
        self.m_sessionInfos = super(SessionEnfReader, self).getSection("SESSION_INFO")

        for key in self.m_sessionInfos:
            if self.m_sessionInfos[key] == "":
                self.m_sessionInfos[key] = None
            elif self.m_sessionInfos[key].lower() == "true":
                self.m_sessionInfos[key]=True
            elif self.m_sessionInfos[key].lower() == "false":
                self.m_sessionInfos[key]=False
            else:
                pass

    def get(self,label):
        if label in self.m_sessionInfos.keys():
            return self.m_sessionInfos[label]

    def getSessionInfos(self):
        return self.m_sessionInfos


class TrialEnfReader(EnfReader):

    def __init__(self, path,enfFile):
        super(TrialEnfReader, self).__init__(path,enfFile)
        self.m_trialInfos = super(TrialEnfReader, self).getSection("TRIAL_INFO")

        for key in self.m_trialInfos:
            if self.m_trialInfos[key] == "" or self.m_trialInfos[key] == "None":
                self.m_trialInfos[key] = None
            elif self.m_trialInfos[key].lower() == "true":
                self.m_trialInfos[key]=True
            elif self.m_trialInfos[key].lower() == "false":
                self.m_trialInfos[key]=False
            else:
                pass

    def set(self,label,value):
        self.m_trialInfos[label] = value
        self.m_config.set('TRIAL_INFO', label, value)


    def save(self):
        with open((self.m_path + self.m_file), 'w') as configfile:
            self.m_config.write(configfile)



    def getTrialInfos(self):
        return self.m_trialInfos

    def get(self,label):
        if label in self.m_trialInfos.keys():
            return self.m_trialInfos[label]

    def getC3d(self):

        return self.m_file.replace(".Trial.enf",".c3d")


    def isSelected(self):
        flag = False
        if "Selected" in self.m_trialInfos.keys():
            if self.m_trialInfos["Selected"] == "Selected" :
                flag = True
        return flag



    def isCalibrationTrial(self):
        flag = False
        if "TrialType" in self.m_trialInfos.keys() and self.m_trialInfos["TrialType"] == "Static":
            flag = True
        return flag

    def isKneeCalibrationTrial(self):
        flag = False
        if "TrialType" in self.m_trialInfos.keys() and self.m_trialInfos["TrialType"] == "Knee Calibration":
            flag = True
        return flag


    def isC3dExist(self):
        return os.path.isfile(self.m_path + self.m_file.replace(".Trial.enf",".c3d"))



    def isMotionTrial(self):
        flag = False
        if "TrialType" in self.m_trialInfos.keys() and self.m_trialInfos["TrialType"] == "Motion":
            flag =  True
        return flag

    def getForcePlateAssigment(self):

        c3dFilename = self.m_file.replace(".Trial.enf",".c3d")
        acq = btkTools.smartReader((self.m_path + c3dFilename))
        nfp = btkTools.getNumberOfForcePlate(acq)

        mfpa = ""
        for i in range(1,nfp+1):
            if self.m_trialInfos["FP"+str(i)]=="Left": mfpa = mfpa +"L"
            if self.m_trialInfos["FP"+str(i)]=="Right": mfpa = mfpa +"R"
            if self.m_trialInfos["FP"+str(i)]=="Invalid": mfpa = mfpa +"X"
            if self.m_trialInfos["FP"+str(i)]=="Auto": mfpa = mfpa +"A"

        return mfpa

    def getMarkerDiameter(self):
        if "MarkerDiameter" in self.m_trialInfos.keys():
            return float(self.m_trialInfos["MarkerDiameter"]) if self.m_trialInfos["MarkerDiameter"] is not None else 14.0

    def getFlatFootOptions(self):
        if "LeftFlatFoot" in self.m_trialInfos.keys():
             leftFlatFoot =  self.m_trialInfos["LeftFlatFoot"]

        if "RightFlatFoot" in self.m_trialInfos.keys():
            RightFlatFoot  = self.m_trialInfos["LeftFlatFoot"]

            return leftFlatFoot, RightFlatFoot

    def isMarked(self):
        markedFiles = getCurrentMarkedEnfs()
        return True if self.m_file in markedFiles else false



class ClassifiedEnf(object):

    def __init__(self,criteria,enfFiles,shortName):
        self.__criteria = criteria
        self.__enfFiles = enfFiles
        self.__shortName = shortName




    def getEnfFiles(self):
        return self.__enfFiles

    def getCriteria(self):
        return self.__criteria

    def getshortName(self):
        return self.__shortName



import logging
import pyCGM2
from pyCGM2 import btk
from pyCGM2.Tools import btkTools
from pyCGM2.Math import derivation,geometry
from pyCGM2.Signal import detect_peaks
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.path import Path
from pyCGM2.Model import model, modelDecorator, frame, motion
from pyCGM2.Model.CGM2 import cgm, cgm2

from pyCGM2.Utils import utils
from pyCGM2.Signal import detect_changes


class GaitEventQualityProcedure(object):
    def __init__(self,acq, title = None):

        self.acq = acq
        self.state = True
        self.exceptionMode = False

        self.title = "Gait events" if title is None else title

    def check(self):

        events = self.acq.GetEvents()

        if events != []:

            events_L = list()
            events_R = list()
            for ev in btk.Iterate(events):
                if ev.GetContext() == "Left":
                    events_L.append(ev)
                if ev.GetContext() == "Right":
                    events_R.append(ev)


            if events_L!=[] and events_R!=[]:
                labels = [it.GetLabel() for it in btk.Iterate(events) if it.GetLabel() in ["Foot Strike","Foot Off"]]
                frames = [it.GetFrame() for it in btk.Iterate(events) if it.GetLabel() in ["Foot Strike","Foot Off"]]

                init = labels[0]
                for i in range(1,len(labels)):
                    label = labels[i]
                    frame = frames[i]
                    if label == init:
                        logging.error("[pyCGM2-Checking] two consecutive (%s) detected at frame (%i)"%((label),frame))
                        if self.exceptionMode:
                            raise Exception("[pyCGM2-Checking]  two consecutive (%s) detected at frame (%i)"%((label),frame))

                        self.state = False
                    init = label

            if events_L!=[]:
                init = events_L[0].GetLabel()
                if len(events_L)>1:
                    for i in range(1,len(events_L)):
                        label = events_L[i].GetLabel()
                        if label == init:
                            logging.error("[pyCGM2-Checking]  Wrong Left Event - two consecutive (%s) detected at frane (%i)"%((label),events_L[i].GetFrame()) )
                            if self.exceptionMode:
                                raise Exception("[pyCGM2-Checking]  Wrong Left Event - two consecutive (%s) detected at frane (%i)"%((label),events_L[i].GetFrame()))

                            self.state = False
                        init = label
                else:
                    logging.warning("Only one left events")
                    self.state = False

            if events_R!=[]:
                init = events_R[0].GetLabel()
                if len(events_R)>1:
                    for i in range(1,len(events_R)):
                        label = events_R[i].GetLabel()
                        if label == init:
                            logging.error("[pyCGM2-Checking] Wrong Right Event - two consecutive (%s) detected at frane (%i)"%((label),events_R[i].GetFrame()) )
                            if self.exceptionMode:
                                raise Exception("[pyCGM2-Checking] Wrong Right Event - two consecutive (%s) detected at frane (%i)"%((label),events_R[i].GetFrame()) )
                            self.state = False
                        init = label
                else:
                    logging.warning("Only one right events ")
                    self.state = False
        else:
            logging.error("[pyCGM2-Checking] No events are in trial")
            if self.exceptionMode:
                raise Exception("[pyCGM2-Checking]  No events are in the trials")
            self.state = False





class AnthropometricDataQualityProcedure(object):
    def __init__(self,mp,title=None):
        self.mp = mp
        self.state = True
        self.exceptionMode = False

        self.title = "CGM anthropometric parameters" if title is None else title

    def check(self):
        """
        TODO :
        - use relation between variable ( width/height)
        - use marker measurement
        """

        if self.mp["RightLegLength"] < 500: logging.warning("[pyCGM2-Checking] Right Leg Lenth < 500 mm");self.state = False
        if self.mp["LeftLegLength"] < 500: logging.warning("[pyCGM2-Checking] Left Leg Lenth < 500 mm");self.state = False
        if self.mp["RightKneeWidth"] < self.mp["RightAnkleWidth"]: logging.error("[pyCGM2-Checking] Right ankle width > knee width ");self.state = False
        if self.mp["LeftKneeWidth"] < self.mp["LeftAnkleWidth"]: logging.error("[pyCGM2-Checking] Right ankle width > knee width ");self.state = False
        if self.mp["RightKneeWidth"] > self.mp["RightLegLength"]: logging.error("[pyCGM2-Checking]  Right knee width > leg length ");self.state = False
        if self.mp["LeftKneeWidth"] > self.mp["LeftLegLength"]: logging.error(" [pyCGM2-Checking] Left knee width > leg length ");self.state = False


        if not utils.isInRange(self.mp["RightKneeWidth"],
            self.mp["LeftKneeWidth"]-0.3*self.mp["LeftKneeWidth"],
            self.mp["LeftKneeWidth"]+0.3*self.mp["LeftKneeWidth"]):
            logging.warning("[pyCGM2-Checking] Knee widths differed by more than 30%")
            self.state = False

        if not utils.isInRange(self.mp["RightAnkleWidth"],
            self.mp["LeftAnkleWidth"]-0.3*self.mp["LeftAnkleWidth"],
            self.mp["LeftAnkleWidth"]+0.3*self.mp["LeftAnkleWidth"]):
            logging.warning("[pyCGM2-Checking] Ankle widths differed by more than 30%")
            self.state = False

        if not utils.isInRange(self.mp["RightLegLength"],
            self.mp["LeftLegLength"]-0.3*self.mp["LeftLegLength"],
            self.mp["LeftLegLength"]+0.3*self.mp["LeftLegLength"]):
            logging.warning("[pyCGM2-Checking] Leg lengths differed by more than 30%")
            self.state = False

class MarkerPresenceQualityProcedure(object):

    def __init__(self,acq,markers=None,title=None):
        self.acq = acq
        self.exceptionMode = False
        self.title = "Marker presence" if title is None else title

        self.markers = markers if markers is not None else btkTools.GetMarkerNames(acq)

        self.state = True
        self.markersIn = []

    def check(self):

        frameNumber = self.acq.GetPointFrameNumber()

        count = 0
        for marker in self.markers:
            try:
                self.acq.GetPoint(marker)
            except RuntimeError:
                logging.warning("[pyCGM2-Checking]  marker [%s] - not exist in the acquisition"%(marker))
            else:
                self.markersIn.append(marker)
                count +=1

        if not count == len(self.markers):
            self.state = False

class GapQualityProcedure(object):

    def __init__(self,acq,markers=None,title=None):
        self.acq = acq
        self.exceptionMode = False

        self.markers = markers if markers is not None else btkTools.GetMarkerNames(acq)
        self.state = True

        self.title = "Gap" if title is None else title

    def check(self):

        frameNumber = self.acq.GetPointFrameNumber()

        for marker in self.markers:
            gapCount = list()
            previousValue = 0
            count =0
            try:
                self.acq.GetPoint(marker)
            except RuntimeError:
                logging.warning("[pyCGM2-Checking] marker [%s] - not exist in the acquisition"%(marker))
            else:
                for i in range(0,frameNumber):
                    val = self.acq.GetPoint(marker).GetResidual(i)
                    if val <0 : count+=1
                    if previousValue<0  and  val==0.0:
                        gapCount.append(count)
                        count = 0
                    previousValue = val

                gapNumber = len(gapCount)
                if gapNumber!=0: maxGap = max(gapCount)

                if gapNumber!=0:
                    logging.warning("[pyCGM2-Checking] marker [%s] - number of gap [%i] and max gap [%i]"%(marker,gapNumber,maxGap))
                    self.state = False






class SwappingMarkerQualityProcedure(object):
    def __init__(self,acq,markers=None,title=None,plot=False):
        self.acq = acq
        self.markers = markers if markers is not None else btkTools.GetMarkerNames(acq)
        self.exceptionMode = False
        self.title = "Swapping marker" if title is None else title

        self.state = True
        self.plot = plot



    def check(self):

        ff = self.acq.GetFirstFrame()
        frameNumber = self.acq.GetPointFrameNumber()
        freq = self.acq.GetPointFrequency()

        for marker in self.markers:
            markerList = list(self.markers)
            markerList.remove(marker)
            nearest,dist = btkTools.findNearestMarker(self.acq,1,marker,markerNames = markerList)
            values = self.acq.GetPoint(marker).GetValues()
            norms = np.linalg.norm(values,axis =1)

            ta, tai, taf, amp = detect_changes.detect_cusum(norms, dist, dist/2.0, True, self.plot)

            if ta.size != 0:
                for index in tai:
                    residual = self.acq.GetPoint(marker).GetResidual(index)
                    if residual>=0.0:
                        frame = index+ff
                        logging.warning("[pyCGM2-Checking] marker [%s] - swapped at frame [%i] (nearest marker= %s - dist=%.2f) "%(marker,frame,nearest,dist))


class MarkerPositionQualityProcedure(object):
    """
    TODO :
    - check medial markers if exist
    """


    def __init__(self,acq,markers = None, title = None):
        self.acq = acq
        self.markers = markers if markers is not None else btkTools.GetMarkerNames(acq)
        self.exceptionMode = False
        self.state = True

        self.title = "Marker position" if title is None else title

    def check(self):

        frameNumber = self.acq.GetPointFrameNumber()


        LASI_values = self.acq.GetPoint("LASI").GetValues()
        RASI_values = self.acq.GetPoint("RASI").GetValues()
        LPSI_values = self.acq.GetPoint("LPSI").GetValues()
        RPSI_values = self.acq.GetPoint("RPSI").GetValues()
        sacrum_values=(self.acq.GetPoint("LPSI").GetValues() + self.acq.GetPoint("RPSI").GetValues()) / 2.0
        midAsis_values=(self.acq.GetPoint("LASI").GetValues() + self.acq.GetPoint("RASI").GetValues()) / 2.0


        projectedLASI = np.array([LASI_values[:,0],LASI_values[:,1],np.zeros((frameNumber))]).T
        projectedRASI = np.array([RASI_values[:,0],RASI_values[:,1],np.zeros((frameNumber))]).T
        projectedLPSI = np.array([LPSI_values[:,0],LPSI_values[:,1],np.zeros((frameNumber))]).T
        projectedRPSI = np.array([RPSI_values[:,0],RPSI_values[:,1],np.zeros((frameNumber))]).T


        for i  in range(0,frameNumber):
            verts = [
                projectedLASI[i,0:2], # left, bottom
                projectedRASI[i,0:2], # left, top
                projectedRPSI[i,0:2], # right, top
                projectedLPSI[i,0:2], # right, bottom
                projectedLASI[i,0:2], # right, top
                ]

            codes = [Path.MOVETO,
                     Path.LINETO,
                     Path.LINETO,
                     Path.LINETO,
                     Path.CLOSEPOLY,
                     ]

            path = Path(verts, codes)

            intersection = geometry.LineLineIntersect(projectedLASI[i,:],projectedLPSI[i,:],projectedRASI[i,:],projectedRPSI[i,:])


            if path.contains_point(intersection[0]):
                logging.error("[pyCGM2-Checking] wrong Labelling of pelvic markers at frame [%i]"%(i))
                if self.exceptionMode:
                    raise Exception("[pyCGM2-Checking] wrong Labelling of pelvic markers at frame [%i]"%(i))

                self.state = False
            else:
                # check marker side
                pt1=RASI_values[i,:]
                pt2=LASI_values[i,:]
                pt3=sacrum_values[i,:]
                ptOrigin=midAsis_values[i,:]

                a1=(pt2-pt1)
                a1=np.divide(a1,np.linalg.norm(a1))
                v=(pt3-pt1)
                v=np.divide(v,np.linalg.norm(v))
                a2=np.cross(a1,v)
                a2=np.divide(a2,np.linalg.norm(a2))

                x,y,z,R=frame.setFrameData(a1,a2,"YZX")

                csFrame_L=frame.Frame()
                csFrame_L.setRotation(R)
                csFrame_L.setTranslation(RASI_values[i,:])

                csFrame_R=frame.Frame()
                csFrame_R.setRotation(R)
                csFrame_R.setTranslation(LASI_values[i,:])


                for marker in self.markers:
                    residual = self.acq.GetPoint(marker).GetResidual(i)

                    if marker[0] == "L":
                        local = np.dot(csFrame_L.getRotation().T,self.acq.GetPoint(marker).GetValues()[i,:]-csFrame_L.getTranslation())
                    if marker[0] == "R":
                        local = np.dot(csFrame_R.getRotation().T,self.acq.GetPoint(marker).GetValues()[i,:]-csFrame_R.getTranslation())
                    if residual >0.0:
                        if marker[0] == "L" and local[1]<0:
                            logging.error("[pyCGM2-Checking] check location of the marker [%s] at frame [%i]"%(marker,i))
                            self.state = False
                            if self.exceptionMode:
                                raise Exception("[pyCGM2-Checking] check location of the marker [%s] at frame [%i]"%(marker,i))

                        if marker[0] == "R" and local[1]>0:
                            logging.error("[pyCGM2-Checking] check location of the marker [%s] at frame [%i]"%(marker,i))
                            self.state = False
                            if self.exceptionMode:
                                raise Exception("[pyCGM2-Checking] check location of the marker [%s] at frame [%i]"%(marker,i))
                                self.state = False


class ForcePlateQualityProcedure(object):

    def __init__(self,acq,title=None):
        self.acq = acq
        self.exceptionMode = False
        self.state = True

        self.title = "Force Plate" if title is None else title

    def check(self):
        # TODO :  - saturation and foot asignment
        pass

class EMGQualityProcedure(object):
    def __init__(self,acq, analogLabels,title=None):
        self.acq = acq
        self.exceptionMode = False
        self.analogLabels =  analogLabels
        self.state = True

        self.title = "EMG" if title is None else title
    def check(self):
        # TODO :  - saturation and foot asignment
        pass



# -*- coding: utf-8 -*-
from enum import Enum


def enumFromtext(memberStr, enum):
    if memberStr  in enum.__members__.keys():
        return enum.__members__[memberStr]
    else:
        raise Exception ("[pyCGM2] %s not found in targeted enum"%(memberStr))


def isValueBelongToEnum(value, enum):
    """

    """
    flag = False
    for it in enum.__members__.items():
        if it[1].value == value:
            flag = True
    return flag




def isEnumMember(member, enum):
    """
        check if member of an enum
    """

    flag = False
    for enumIt in enum:
           if enumIt == member:
                flag = True
    return flag


class DataType(Enum):
    """  """
    Marker = 0
    Angle = 1
    Segment = 3
    Moment = 4
    Force = 5
    Power = 6


class motionMethod(Enum):
    """ Enum defining method uses for computing a segment pose """
    Unknown = 0
    Determinist = 1
    Sodervisk = 2


class MomentProjection(Enum):
    """ Enum defining in which Segment expressed kinetics"""
    Global = "Global"
    Proximal = "Proximal"
    Distal = "Distal"
    JCS = "JCS"
    JCS_Dual ="JCS_Dual"


class HarringtonPredictor(Enum):
    """ Enum defining harrington's regression predictor"""
    Native = "full"
    PelvisWidth = "PWonly"
    LegLength = "LLonly"

class SegmentSide(Enum):
    """ Enum defining segment side"""
    Central = 0
    Left = 1
    Right = 2


class EmgAmplitudeNormalization(Enum):
    """ Enum defining harrington's regression predictor"""
    MaxMax = "MaxMax"
    MeanMax = "MeanMax"
    MedianMax = "MedianMax"
    Threshold = "Threshold"

class BodyPart(Enum):
    LowerLimb=0
    LowerLimbTrunk=1
    FullBody=2
    UpperLimb=3


class JointCalibrationMethod(Enum):
    Basic = "lateralMarker"
    KAD = "KAD"
    Medial = "medial"

class BodyPartPlot(Enum):
    LowerLimb="LowerLimb"
    Trunk="Trunk"
    UpperLimb="UpperLimb"

class EclipseType(Enum):
    Session="Session.enf"
    Trial="Trial.enf"
    Patient="Patient.enf"

class AnalysisSection(Enum):
    Kinematic="Kinematic"
    Kinetic="Kinetic"
    Emg="Emg"
# --- enum used with Btk-Models
# obsolete
#class BspModel(Enum):
#    Dempster = "Dempster"
#    DempsterVicon = "DempsterVicon"
#    DeLeva = "DeLeva"
#
#class Sex(Enum):
#    Male = "M"
#    Female = "F"
#
#
#class InverseDynamicAlgo(Enum):
#    Quaternion = "quaternion"
#    Generic = "generic"
#    RotationMatrix = "rotationMatrix"



# -*- coding: utf-8 -*-
"""
Created on Mon Oct 10 11:47:01 2016

@author: Fabien Leboeuf ( Salford Univ, UK)
"""

import numpy as np


def angleFrom2Vectors(v1,v2, vn = None):
    """
        Return a signed angle between 2 vectors

        .. note:: the common orthogonal vector is used for defining the sign of the angle

        :Parameters:
            - `v1` (numpy.array(3,)) - first vector
            - `v2` (numpy.array(3,)) - second vector
            - `vn` (numpy.array(3,)) - common orthogonal vector.

        :Return:
            - `angle` (double) - signed angle

    """


    cross = np.cross(v1, v2)
    angle= np.arctan2(np.linalg.norm(np.cross(v1, v2)), np.dot(v2, v1))

    # if vn is not None:
    #     if (np.dot(vn, cross) < 0):
    #         angle = -angle;

    return angle

def oppositeVector(v1):
    return (-1*np.ones(3))*v1

def LineLineIntersect(p1,p2,p3,p4):
    """
        Calculates the line segment pa_pb that is the shortest route
        between two lines p1_p2 and p3_p4. Calculates also the values of
        mua and mub where:
            - pa = p1 + mua (p2 - p1)
            - pb = p3 + mub (p4 - p3)

        :parameters:
            - p1 ('np.array(3)') : 3d coordinates
            - p2 ('np.array(3)') : 3d coordinates
            - p3 ('np.array(3)') : 3d coordinates
            - p4 ('np.array(3)') : 3d coordinates

        .. note::

            this a python conversion from the code proposed by
            Paul Bourke at http://astronomy.swin.edu.au/~pbourke/geometry/lineline3d/

    """
    pa = np.zeros((3))
    pb = np.zeros((3))

    p13 = np.zeros((3))
    p43 = np.zeros((3))


    p13[0] = p1[0] - p3[0]
    p13[1] = p1[1] - p3[1]
    p13[2] = p1[2] - p3[2]

    p43[0] = p4[0] - p3[0]
    p43[1] = p4[1] - p3[1]
    p43[2] = p4[2] - p3[2]

    if ((np.abs(p43[0])  <  np.spacing(1)) and  (np.abs(p43[1])  <  np.spacing(1)) and (np.abs(p43[2])  <  np.spacing(1))):
        raise Exception(" [pyCGM2] Could not compute LineLineIntersect! ")

    p21 = np.zeros((3))

    p21[0] = p2[0] - p1[0];
    p21[1] = p2[1] - p1[1];
    p21[2] = p2[2] - p1[2];

    if ((np.abs(p21[0])  < np.spacing(1)) and (np.abs(p21[1])  < np.spacing(1)) and  (np.abs(p21[2])  < np.spacing(1))):
        raise Exception(" [pyCGM2] Could not compute LineLineIntersect! ")

    d1343 = p13[0] * p43[0] + p13[1] * p43[1] + p13[2] * p43[2];
    d4321 = p43[0] * p21[0] + p43[1] * p21[1] + p43[2] * p21[2];
    d1321 = p13[0] * p21[0] + p13[1] * p21[1] + p13[2] * p21[2];
    d4343 = p43[0] * p43[0] + p43[1] * p43[1] + p43[2] * p43[2];
    d2121 = p21[0] * p21[0] + p21[1] * p21[1] + p21[2] * p21[2];

    denom = d2121 * d4343 - d4321 * d4321;

    if (np.abs(denom) < np.spacing(1)):
        raise Exception(" [pyCGM2] Could not compute LineLineIntersect! ")

    numer = d1343 * d4321 - d1321 * d4343;

    mua = numer / denom;
    mub = (d1343 + d4321 * mua) / d4343;

    pa[0] = p1[0] + mua * p21[0];
    pa[1] = p1[1] + mua * p21[1];
    pa[2] = p1[2] + mua * p21[2];
    pb[0] = p3[0] + mub * p43[0];
    pb[1] = p3[1] + mub * p43[1];
    pb[2] = p3[2] + mub * p43[2];

    return pa,pb



# -*- coding: utf-8 -*-
import pyCGM2
from pyCGM2.Configurator import Manager
from pyCGM2.Utils import files
from pyCGM2 import enums
import logging
import copy
from pyCGM2.Eclipse import vskTools,eclipse
import os

class ModelConfigManager(Manager.ConfigManager):
    """

    """
    def __init__(self,settings):
        super(ModelConfigManager, self).__init__(settings)


    @property
    def staticTrial(self):
        return  self._userSettings["Calibration"]["StaticTrial"]

    @property
    def dynamicTrials(self):
        return  self._userSettings["Fitting"]["Trials"]

    @property
    def listOfdynamicTrials(self):
        li =list()
        for it in self._userSettings["Fitting"]["Trials"]:
            li.append(it["File"])

        return li

    def contruct(self):

        finalSettings =  copy.deepcopy(self._internSettings)

        finalSettings["Calibration"].update({"StaticTrial":self._userSettings["Calibration"]["StaticTrial"]})
        finalSettings["Fitting"].update({"Trials":self._userSettings["Fitting"]["Trials"]})

        for key in self._userSettings.keys(): #upate of #mp
            if key  not in finalSettings.keys():
                finalSettings.update({key : self._userSettings[key]})

        self.finalSettings = finalSettings




class CGM1ConfigManager(ModelConfigManager):
    """

    """
    def __init__(self,settings,localInternalSettings = None, localTranslators=None,vsk=None):
        super(CGM1ConfigManager, self).__init__(settings)

        self._localTranslators = localTranslators
        self._localInternalSettings = localInternalSettings

        self.__internalsettings()

        # run data to overwrite in internalSettings
        self.leftFlatFoot
        self.rightFlatFoot
        self.headFlat
        self.markerDiameter
        self.pointSuffix
        self.translators
        self.momentProjection

        if vsk is not None:
            self._vsk = vsk
        else:
            self._vsk = None

        self.requiredMp
        self.optionalMp


    def __internalsettings(self):
        if self._localInternalSettings is None:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM1-pyCGM2.settings"):
                self._internSettings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM1-pyCGM2.settings")
            else:
                self._internSettings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM1-pyCGM2.settings")
        else:
            logging.info("Local internal setting found")
            self._internSettings = self._localInternalSettings


    @property
    def leftFlatFoot(self):
        value = self._userSettings["Calibration"]["Left flat foot"]
        self._internSettings["Calibration"]["Left flat foot"] = value # overwriting
        return value

    @property
    def rightFlatFoot(self):
        value = self._userSettings["Calibration"]["Right flat foot"]
        self._internSettings["Calibration"]["Right flat foot"] = value # overwriting
        return value

    @property
    def headFlat(self):
        value = self._userSettings["Calibration"]["Head flat"]
        self._internSettings["Calibration"]["Head flat"] = value # overwriting
        return value

    @property
    def markerDiameter(self):
        value = self._userSettings["Global"]["Marker diameter"]
        self._internSettings["Global"]["Marker diameter"] = value # overwriting
        return  value

    @property
    def pointSuffix(self):
        value = self._userSettings["Global"]["Point suffix"]
        self._internSettings["Global"]["Point suffix"] = value # overwriting

        return  None if value=="None" else value

    @property
    def translators(self): # overwriting if Translators exist
        if self._localTranslators is not None:
            translators = self._localTranslators
            self._internSettings["Translators"] = translators["Translators"]
            return self._internSettings["Translators"]
        else:
            return self._internSettings["Translators"]

    @property
    def momentProjection(self):
        if self._internSettings["Fitting"]["Moment Projection"] == "Distal":
            return  enums.MomentProjection.Distal
        elif self._internSettings["Fitting"]["Moment Projection"] == "Proximal":
            return  enums.MomentProjection.Proximal
        elif self._internSettings["Fitting"]["Moment Projection"] == "Global":
            return  enums.MomentProjection.Global
        elif self._internSettings["Fitting"]["Moment Projection"] == "JCS":
            return enums.MomentProjection.JCS

    @property
    def requiredMp(self):
        if self._vsk is None:
            return  self._userSettings["MP"]["Required"]
        else:
            required_mp,optional_mp = vskTools.getFromVskSubjectMp(self._vsk, resetFlag=True)
            self._userSettings["MP"]["Required"].update(required_mp)
            return required_mp

    @property
    def optionalMp(self):
        if self._vsk is None:
            return  self._userSettings["MP"]["Optional"]
        else:
            required_mp,optional_mp = vskTools.getFromVskSubjectMp(self._vsk, resetFlag=True)
            self._userSettings["MP"]["Optional"].update(optional_mp)
            return optional_mp



    def updateMp(self,model):

        if self.finalSettings is not None:
            self.finalSettings["MP"]["Required"].update(model.mp)
            self.finalSettings["MP"]["Optional"].update(model.mp_computed)



class CGM1_1ConfigManager(CGM1ConfigManager):
    """

    """
    def __init__(self,settings,localInternalSettings = None, localTranslators=None,vsk=None):
        super(CGM1_1ConfigManager, self).__init__(settings,localInternalSettings = localInternalSettings, localTranslators=localTranslators, vsk=vsk)

    def __internalsettings(self):
        if self._localInternalSettings is None:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM1_1-pyCGM2.settings"):
                self._internSettings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM1_1-pyCGM2.settings")
            else:
                self._internSettings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM1_1-pyCGM2.settings")

        else:
            self._internSettings = self._localInternalSettings

class CGM2_1ConfigManager(CGM1ConfigManager):
    """

    """
    def __init__(self,settings,localInternalSettings = None, localTranslators=None,vsk=None):
        super(CGM2_1ConfigManager, self).__init__(settings,localInternalSettings = localInternalSettings, localTranslators=localTranslators,vsk=vsk)

        self.__internalsettings()

    def __internalsettings(self):
        if self._localInternalSettings is None:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_1-pyCGM2.settings"):
                self._internSettings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_1-pyCGM2.settings")
            else:
                self._internSettings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_1-pyCGM2.settings")
        else:
            self._internSettings = self._localInternalSettings


    @property
    def hjcMethod(self):
        return self._internSettings["Calibration"]["HJC"]

class CGM2_2ConfigManager(CGM2_1ConfigManager):
    """

    """
    def __init__(self,settings,localInternalSettings = None, localTranslators=None, localIkWeight=None,vsk=None):
        super(CGM2_2ConfigManager, self).__init__(settings,localInternalSettings = localInternalSettings, localTranslators=localTranslators,vsk=vsk)

        self._localIkweight = localIkWeight

        self.__internalsettings()

        # run data to overwrite in internalSettings
        self.ikWeight

    def __internalsettings(self):
        if self._localInternalSettings is None:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_2-pyCGM2.settings"):
                self._internSettings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_2-pyCGM2.settings")
            else:
                self._internSettings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_2-pyCGM2.settings")
        else:
            self._internSettings = self._localInternalSettings



    @property
    def hjcMethod(self):
        return self._internSettings["Calibration"]["HJC"]


    @property
    def ikWeight(self): # overwriting if Translators exist
        if self._localIkweight is not None:
            self._internSettings["Fitting"]["Weight"] = self._localIkweight
            return self._internSettings["Fitting"]["Weight"]
        else:
            return self._internSettings["Fitting"]["Weight"]

class CGM2_3ConfigManager(CGM2_2ConfigManager):
    """

    """
    def __init__(self,settings,localInternalSettings = None, localTranslators=None, localIkWeight=None,vsk=None):
        super(CGM2_3ConfigManager, self).__init__(settings,localInternalSettings = localInternalSettings, localTranslators=localTranslators,
                                                localIkWeight=localIkWeight,vsk=vsk)

        self.__internalsettings()

    def __internalsettings(self):
        if self._localInternalSettings is None:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_3-pyCGM2.settings"):
                self._internSettings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_3-pyCGM2.settings")
            else:
                self._internSettings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_3-pyCGM2.settings")
        else:
            self._internSettings = self._localInternalSettings

    @property
    def enableIK(self):
        return self._internSettings["Global"]["EnableIK"]

class CGM2_4ConfigManager(CGM2_3ConfigManager):
    """

    """
    def __init__(self,settings,localInternalSettings = None, localTranslators=None, localIkWeight=None,vsk=None):
        super(CGM2_4ConfigManager, self).__init__(settings,localInternalSettings = localInternalSettings, localTranslators=localTranslators,localIkWeight=localIkWeight,vsk=vsk)

        self.__internalsettings()

    def __internalsettings(self):
        if self._localInternalSettings is None:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_4-pyCGM2.settings"):
                self._internSettings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_4-pyCGM2.settings")
            else:
                self._internSettings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_4-pyCGM2.settings")

        else:
            self._internSettings = self._localInternalSettings


class CGM2_5ConfigManager(CGM2_4ConfigManager):
    """

    """
    def __init__(self,settings,localInternalSettings = None, localTranslators=None, localIkWeight=None,vsk=None):
        super(CGM2_5ConfigManager, self).__init__(settings,localInternalSettings = localInternalSettings, localTranslators=localTranslators,localIkWeight=localIkWeight,vsk=vsk)

        self.__internalsettings()

    def __internalsettings(self):
        if self._localInternalSettings is None:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_5-pyCGM2.settings"):
                self._internSettings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_5-pyCGM2.settings")
            else:
                self._internSettings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_5-pyCGM2.settings")
        else:
            self._internSettings = self._localInternalSettings



# -*- coding: utf-8 -*-
import numpy as np
import logging
from pyCGM2 import enums
from pyCGM2.Model import modelFilters, modelDecorator



def applyBasicDecorators(dcm, model,acqStatic,optional_mp,markerDiameter,cgm1only=False):
    if model.getBodyPart() != enums.BodyPart.UpperLimb:
        # native but thighRotation altered in mp
        if dcm["Left Knee"] == enums.JointCalibrationMethod.Basic and  dcm["Left Ankle"] == enums.JointCalibrationMethod.Basic and optional_mp["LeftThighRotation"] !=0:
            logging.warning("CASE FOUND ===> Left Side = NATIVE CGM1 + manual Thigh  ")
            modelDecorator.Cgm1ManualOffsets(model).compute(acqStatic,"left",optional_mp["LeftThighRotation"],
                markerDiameter, optional_mp["LeftTibialTorsion"], optional_mp["LeftShankRotation"])

        if dcm["Right Knee"] == enums.JointCalibrationMethod.Basic and  dcm["Right Ankle"] == enums.JointCalibrationMethod.Basic and optional_mp["RightThighRotation"] !=0:
            logging.warning("CASE FOUND ===> Right Side = NATIVE CGM1 + manual Thigh  ")
            modelDecorator.Cgm1ManualOffsets(model).compute(acqStatic,"right",optional_mp["RightThighRotation"],
                markerDiameter,optional_mp["RightTibialTorsion"],optional_mp["RightShankRotation"])

        # KAD - and Kadmed
        if dcm["Left Knee"] == enums.JointCalibrationMethod.KAD:
            logging.warning("CASE FOUND ===> Left Side = Knee-KAD")
            modelDecorator.Kad(model,acqStatic).compute(markerDiameter=markerDiameter, side="left")
            if  dcm["Left Ankle"] == enums.JointCalibrationMethod.Medial:
                logging.warning("CASE FOUND ===> Left Side = Ankle-Med")
                modelDecorator.AnkleCalibrationDecorator(model).midMaleolus(acqStatic, markerDiameter=markerDiameter, side="left")

        if dcm["Right Knee"] == enums.JointCalibrationMethod.KAD:
            logging.warning("CASE FOUND ===> Right Side = Knee-KAD")
            modelDecorator.Kad(model,acqStatic).compute(markerDiameter=markerDiameter, side="right")
            if  dcm["Right Ankle"] == enums.JointCalibrationMethod.Medial:
                logging.warning("CASE FOUND ===> Right Side = Ankle-Med")
                modelDecorator.AnkleCalibrationDecorator(model).midMaleolus(acqStatic, markerDiameter=markerDiameter, side="right")

    if not cgm1only:

        if model.getBodyPart() != enums.BodyPart.UpperLimb:
            #Kad-like (KneeMed)
            if dcm["Left Knee"] == enums.JointCalibrationMethod.Medial and dcm["Left Ankle"] == enums.JointCalibrationMethod.Basic:
                modelDecorator.KneeCalibrationDecorator(model).midCondyles_KAD(acqStatic, markerDiameter=markerDiameter, side="left")
            if dcm["Right Knee"] == enums.JointCalibrationMethod.Medial and dcm["Right Ankle"] == enums.JointCalibrationMethod.Basic:
                modelDecorator.KneeCalibrationDecorator(model).midCondyles_KAD(acqStatic, markerDiameter=markerDiameter, side="right")

            #knee and ankle Med
            if dcm["Left Knee"] == enums.JointCalibrationMethod.Medial and dcm["Left Ankle"] == enums.JointCalibrationMethod.Medial:
                logging.info("[pyCGM2] Left Knee : Medial - Left Ankle : Medial")
                modelDecorator.KneeCalibrationDecorator(model).midCondyles(acqStatic, markerDiameter=markerDiameter, side="left")
                modelDecorator.AnkleCalibrationDecorator(model).midMaleolus(acqStatic, markerDiameter=markerDiameter, side="left")

            if dcm["Right Knee"] == enums.JointCalibrationMethod.Medial and dcm["Right Ankle"] == enums.JointCalibrationMethod.Medial:
                logging.info("[pyCGM2] Right Knee : Medial - Right Ankle : Medial")
                modelDecorator.KneeCalibrationDecorator(model).midCondyles(acqStatic, markerDiameter=markerDiameter, side="right")
                modelDecorator.AnkleCalibrationDecorator(model).midMaleolus(acqStatic, markerDiameter=markerDiameter, side="right")


def applyHJCDecorators(model,method):
    if model.getBodyPart() != enums.BodyPart.UpperLimb:
        if method["Left"] == "Hara":
            logging.info("[pyCGM2] Left HJC : Hara")
            modelDecorator.HipJointCenterDecorator(model).hara(side = "left")
        elif len(method["Left"]) == 3:
            logging.info("[pyCGM2] Left HJC : Custom")
            logging.warning(method["Left"])
            modelDecorator.HipJointCenterDecorator(model).custom(position_Left =  np.array(method["Left"]), methodDesc = "custom",side="left")

        if method["Right"] == "Hara":
            logging.info("[pyCGM2] Right HJC : Hara")
            modelDecorator.HipJointCenterDecorator(model).hara(side = "right")
        elif len(method["Right"]) == 3:
            logging.info("[pyCGM2] Right HJC : Custom")
            logging.warning(method["Right"])
            modelDecorator.HipJointCenterDecorator(model).custom(position_Right =  np.array(method["Right"]), methodDesc = "custom",side="right")
